﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using EveMapControls;
using EVEMapNG.SubForms;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.EveAPI;
using libEveMapNG.Filtering;
using libEveMapNG.Utility;
using Region = libEveMapNG.Data.Static.Region;

namespace EVEMapNG
{
    public partial class MainWindow : Form
    {
        //Events

        #region Delegates

        public delegate void SelectedSystemChangedHandler(SolarSystem system, EventArgs e);

        #endregion

        //Application & User Settings

        private readonly RouteWebServer _routeWebServer;
        private readonly SystemSelectBox _systemSelectBox;
        public SelectedSystemChangedHandler SystemChanged;

        public MainWindow()
        {
            InitializeComponent();
            mapView.InitializeMapSettings();
            cboMainZoom.SelectedIndex = 9;
            ssbMain.SystemSelectControl.SelectionChangeCommitted += tsSSB_FindSystem;
            mapView.MouseWheel += Viewport_MouseWheel;
            mapView.SystemSelected += SolarSystemSelected;

            Text = Assembly.GetExecutingAssembly().GetName().Name + " " +
                   Assembly.GetExecutingAssembly().GetName().Version;

            _systemSelectBox = new SystemSelectBox();
            _systemSelectBox.SelectionChangeCommitted += ToolbarSystemSelected;
            _routeWebServer = new RouteWebServer();
            tsmSettingsShowRoutesInWebpage.Enabled = _routeWebServer.IsRunning;
            jumpPlanner.Viewport = mapView;
            manualJumpPlanner.Viewport = mapView;
            gatePlanner1.Viewport = mapView;
            RestoreState();
        }

        public SolarSystem SelectedSolarSystem
        {
            get { return mapView.SelectedSystem; }
            set
            {
                mapView.SelectedSystem = value;
                SolarSystemSelected();
                SystemChanged(SelectedSolarSystem, null);
            }
        }

        private void RestoreState()
        {
            if (UserSettings.Instance.AppState.MWState == null)
                UserSettings.Instance.AppState.LoadDefaultSettings();

            tbOptions.SelectedIndex = UserSettings.Instance.AppState.MWState.SelectedTab;
            cboMainZoom.SelectedIndex = UserSettings.Instance.AppState.MWState.ZoomLevel;
            Location = UserSettings.Instance.AppState.MWState.WindowLocation;
            Size = UserSettings.Instance.AppState.MWState.WindowSize;
            WindowState = UserSettings.Instance.AppState.MWState.WindowState;

            if (UserSettings.Instance.AppState.MWState.SelectedSystem != 0)
                mapView.SelectedSystem = SolarSystem.GetByID(UserSettings.Instance.AppState.MWState.SelectedSystem);

            mapView.PointX = UserSettings.Instance.AppState.MWState.X;
            mapView.PointY = UserSettings.Instance.AppState.MWState.Y;

            //restore control states
            //gate
            gatePlanner1.RestoreState();

            //jump
            jumpPlanner.RestoreState();

            //manual
            manualJumpPlanner.RestoreState();
        }

        private void tbOptions_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tbOptions.SelectedTab == tabFilter) mapView.FilterType = MapViewport.SystemFilterType.UserFilter;
            if (tbOptions.SelectedTab == tabDetails) mapView.FilterType = MapViewport.SystemFilterType.None;
            if (tbOptions.SelectedTab == tabJumpRoute) mapView.FilterType = MapViewport.SystemFilterType.JumpRange;
            mapView.Invalidate();
        }

        private void SaveState()
        {
            UserSettings.Instance.AppState.MWState.SelectedTab = tbOptions.SelectedIndex;
            UserSettings.Instance.AppState.MWState.ZoomLevel = cboMainZoom.SelectedIndex;
            UserSettings.Instance.AppState.MWState.WindowLocation = Location;
            UserSettings.Instance.AppState.MWState.WindowSize = Size;
            UserSettings.Instance.AppState.MWState.WindowState = WindowState;

            if (mapView.SelectedSystem != null)
                UserSettings.Instance.AppState.MWState.SelectedSystem = mapView.SelectedSystem.ID;
            UserSettings.Instance.AppState.MWState.X = mapView.PointX;
            UserSettings.Instance.AppState.MWState.Y = mapView.PointY;

            //save control states
            //gate
            gatePlanner1.SaveState();

            //jump
            jumpPlanner.SaveState();

            //manual
            manualJumpPlanner.SaveState();
        }

        private void Viewport_MouseWheel(object sender, MouseEventArgs e)
        {
            const int mouseResolution = 120; // built-in delta count for each "click" up or down of the wheel
            int clicks = e.Delta/mouseResolution;

            if (e.Delta > 0 && cboMainZoom.SelectedIndex < cboMainZoom.Items.Count - clicks) // zoom in
                cboMainZoom.SelectedIndex += clicks;
            else if (e.Delta < 0 && cboMainZoom.SelectedIndex >= -clicks) // zoom out
                cboMainZoom.SelectedIndex += clicks;
        }


        private void tsSSB_FindSystem(object sender, EventArgs e)
        {
            mapView.CenterOnPoint(ssbMain.SystemSelectControl.SelectedSystem.OrganicCoordinates);
            mapView.SelectedSystem = ssbMain.SystemSelectControl.SelectedSystem;
        }

        private void SolarSystemSelected()
        {
            SolarSystem system = mapView.SelectedSystem;
            if (system == null)
                return;

            if (tbOptions.SelectedTab == tabManualJumpPlanner)
            {
                manualJumpPlanner.UpdateSystemsInJumpRange(system);
            }

            mapView.Invalidate();

            details1.CurrentSystem = SelectedSolarSystem;
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            if (-1 < cboMainZoom.SelectedIndex - 1)
                cboMainZoom.SelectedIndex--;
        }

        private void ts_ZoomPlus_Click(object sender, EventArgs e)
        {
            if (cboMainZoom.Items.Count > cboMainZoom.SelectedIndex + 1)
                cboMainZoom.SelectedIndex++;
        }

        private void updateStarbasesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApiWorker myWorker = new ApiWorker(ApiFetch.Starbase);
            myWorker.Show();
        }

        private void tsmDetailOptionsCheckChanged(object sender, EventArgs e)
        {
            //Store a list of all the detail options so we dont have to use reflection or whatever Control.Find() is
            List<ToolStripMenuItem> menuItems = new List<ToolStripMenuItem>
                                                    {
                                                        tsmDetailNone,
                                                        tsmDetailBelt,
                                                        tsmDetailIceField,
                                                        tsmDetailOre,
                                                        tsmDetailStation,
                                                        tsmDetailMoon,
                                                        tsmDetailSov,
                                                        tsmDetailRat,
                                                        tsmDetailKill,
                                                        tsmDetailJump,
                                                        tsmStarbaseStatus
                                                    };
            ToolStripMenuItem checkedControl = null;
            foreach (ToolStripMenuItem item in menuItems)
            {
                if (sender == item)
                {
                    item.Checked = true;
                    checkedControl = (ToolStripMenuItem) sender;
                }
                else
                    item.Checked = false;
            }

            if (sender == tsmDetailSov || sender == tsmDetailJump || sender == tsmDetailKill)
            {
                //Wait 31 minutes between refreshes to try to avoid making CCP mad.
                if ((DynamicStore.Instance.ApiData.ApiLastRefresh + TimeSpan.FromMinutes(31)) > DateTime.Now)
                {
                    DialogResult result =
                        MessageBox.Show("EVE API Data is out of date or does not exist.  Would you like to refresh?",
                                        "Refresh API Data", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question,
                                        MessageBoxDefaultButton.Button3);
                    switch (result)
                    {
                        case DialogResult.Yes:
                            ApiWorker myWorker = new ApiWorker(ApiFetch.Full);
                            myWorker.Show();
                            break;
                        default:
                            if (checkedControl != null) checkedControl.Checked = false;
                            return;
                    }
                }
            }

            if (tsmDetailNone.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.None;
            if (tsmDetailBelt.Checked)
                UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.AsteroidBelts;
            if (tsmDetailIceField.Checked)
                UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.IceFields;
            if (tsmDetailOre.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.OreClass;
            if (tsmDetailStation.Checked)
                UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Stations;
            if (tsmDetailMoon.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Moons;
            if (tsmDetailSov.Checked)
                UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Sovereignty;
            if (tsmDetailRat.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Rats;
            if (tsmDetailKill.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Kills;
            if (tsmDetailJump.Checked) UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.Jumps;
            if (tsmStarbaseStatus.Checked)
                UserSettings.Instance.ActiveViewportStyle.DetailText = EVEMapDetailText.StarbaseStatus;
            mapView.Invalidate();
        }

        private void ssbMain_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                SolarSystem centerOnSystem = SolarSystem.GetByName(ssbMain.Text);
                if (centerOnSystem != null)
                    mapView.CenterOnPoint(centerOnSystem.OrganicCoordinates);
            }
        }

        private void tsmSettingsShowRoutesInWebpage_Click(object sender, EventArgs e)
        {
            Process.Start(RouteWebServer.ServerAddress);
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Settings mySettings = new Settings();
            mySettings.Show();
            mapView.ViewOptionsMapVisualsChanged(true, true);
            mapView.Invalidate();
        }

        private void MapViewLoad(object sender, EventArgs e)
        {
        }

        #region Context Menu

        private void setSourceToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tbOptions.SelectedTab == tabGateRoute)
            {
                gatePlanner1.SetRouteStart(mapView.HoverSystem);
            }
            if (tbOptions.SelectedTab == tabJumpRoute)
            {
                jumpPlanner.SetJumpStart(mapView.HoverSystem);
            }
        }

        private void cmSystemSelectMenu_Opening(object sender, CancelEventArgs e)
        {
            if (mapView.SelectedSystem == null)
            {
                tsmContextAddCyno.Enabled = false;
                return;
            }
            if (mapView.SelectedSystem.Security < 0)
            {
                if (!DynamicStore.Instance.CynoBeacons.ContainsKey(mapView.SelectedSystem))
                    tsmContextAddCyno.Enabled = true;
            }
        }

        private void setDestinationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tbOptions.SelectedTab == tabGateRoute)
            {
                gatePlanner1.SetRouteEnd(mapView.HoverSystem);
            }
            if (tbOptions.SelectedTab == tabJumpRoute)
            {
                jumpPlanner.SetJumpEnd(mapView.HoverSystem);
            }
        }

        private void markSystemToAvoidToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tbOptions.SelectedTab == tabGateRoute)
                gatePlanner1.AvoidSystem(mapView.HoverSystem);
            if (tbOptions.SelectedTab == tabJumpRoute)
                jumpPlanner.AddJumpAvoid(mapView.HoverSystem);
        }

        private void addWaypointToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (tbOptions.SelectedTab == tabGateRoute)
                gatePlanner1.AddWaypoint(mapView.HoverSystem);
            if (tbOptions.SelectedTab == tabJumpRoute)
                jumpPlanner.AddJumpWaypoint(mapView.HoverSystem);
        }

        private void addCynoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CynoBeacon myBeacon = new CynoBeacon
                                      {
                                          System = mapView.SelectedSystem
                                      };

            if (!DynamicStore.Instance.CynoBeacons.ContainsKey(myBeacon.System))
            {
                DynamicStore.Instance.CynoBeacons.Add(mapView.SelectedSystem, myBeacon);
                DynamicStore.WriteToFile();
                mapView.Invalidate();
            }
        }

        #endregion

        #region Menu Bar

        private void cmbZoom_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cboMainZoom.SelectedIndex)
            {
                case 0:

                    mapView.MapScale = 0.10001f;
                    break;
                case 1:
                    mapView.MapScale = 0.20001f;
                    break;
                case 2:
                    mapView.MapScale = 0.30001f;
                    break;
                case 3:
                    mapView.MapScale = 0.40001f;
                    break;
                case 4:
                    mapView.MapScale = 0.50001f;
                    break;
                case 5:
                    mapView.MapScale = 0.60001f;
                    break;
                case 6:
                    mapView.MapScale = 0.70001f;
                    break;
                case 7:
                    mapView.MapScale = 0.80001f; // avoid randomness when comparing to 0.8f in the map code
                    break;
                case 8:
                    mapView.MapScale = 0.90001f;
                    break;
                case 9:
                    mapView.MapScale = 1.0001f;
                    break;
                case 10:
                    mapView.MapScale = 1.10001f;
                    break;
                case 11:
                    mapView.MapScale = 1.20001f;
                    break;
                case 12:
                    mapView.MapScale = 1.30001f;
                    break;
                case 13:
                    mapView.MapScale = 1.49999f;
                    break;
            }
        }

        private void importJumpBridgesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageJumpbridges jumpbridgeForm = new ManageJumpbridges();
            jumpbridgeForm.ShowDialog();
            jumpbridgeForm.Dispose();
            mapView.Invalidate();
        }

        private void importCynoBeaconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageCynoArrays cynoForm = new ManageCynoArrays();
            cynoForm.ShowDialog();
            cynoForm.Dispose();
            mapView.Invalidate();
        }

        private void aboutEVEMapNGToolStripMenuItem_Click(object sender, EventArgs e)
        {
            About myAbout = new About();
            myAbout.ShowDialog();
        }

        #endregion

        #region Form Save/Load

        private void MainWindow_Load(object sender, EventArgs e)
        {
            //Force form on top when we're ready for the user.
            BringToFront();
            Focus();
        }


        private void ToolbarSystemSelected(object sender, EventArgs e)
        {
            SolarSystem system = ((SystemSelectBox) sender).SelectedItem as SolarSystem;
            if (system == null) return;
            SolarSystemSelected();
            mapView.CenterOnPoint(system.OrganicCoordinates);
        }

        private void MainWindow_FormClosed(object sender, FormClosedEventArgs e)
        {
            SaveState();

            UserSettings.Save();

            _routeWebServer.Dispose();
        }

        #endregion

        #region API Data Menu

        private void updateEverythingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApiWorker myWorker = new ApiWorker(ApiFetch.Full);
            myWorker.Show();
            mapView.Invalidate();
        }

        private void updateSovereigntyDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApiWorker myWorker = new ApiWorker(ApiFetch.Sovereignty);
            myWorker.Show();
            mapView.Invalidate();
        }

        private void updateAllianceDataToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApiWorker myWorker = new ApiWorker(ApiFetch.Alliance);
            myWorker.Show();
            mapView.Invalidate();
        }

        private void updateRecentJumpsKillsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ApiWorker myWorker = new ApiWorker(ApiFetch.MapKills);
            myWorker.Show();
            mapView.Invalidate();
        }

        #endregion

        #region Filter tab handlers

        // Filters
        private readonly FilterList mainFilterList = new FilterList();

        private void cbbFilterGroup_SelectedIndexChanged(object sender, EventArgs e)
        {
            cbbFilterProperty.Text = "";
            cbbFilterProperty.Items.Clear();
            cbbFilterCondition.Text = "";
            cbbFilterCondition.Items.Clear();
            cbbFilterValue.Text = "";
            cbbFilterValue.Items.Clear();
            tbFilterValue.Clear();

            cbbFilterProperty.Enabled = true;
            cbbFilterCondition.Enabled = false;
            cbbFilterValue.Enabled = false;
            tbFilterValue.Enabled = false;
            btnFilterAdd.Enabled = false;

            switch (cbbFilterGroup.SelectedIndex)
            {
                case 0: // NPC Faction
                    cbbFilterProperty.Items.Add("Faction Name");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 1: // NPC Corporation
                    cbbFilterProperty.Items.Add("Corporation Name");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 2: // Agent
                    cbbFilterProperty.Items.Add("Level");
                    cbbFilterProperty.Items.Add("Quality");
                    cbbFilterProperty.Items.Add("Locator");
                    cbbFilterProperty.Items.Add("Division");
                    cbbFilterProperty.Items.Add("Type");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 3: // Station
                    cbbFilterProperty.Items.Add("Reprocessing");
                    cbbFilterProperty.Items.Add("Manufacturing");
                    cbbFilterProperty.Items.Add("Cloning");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 4: // Region
                    cbbFilterProperty.Items.Add("Region Name");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 5: // System
                    cbbFilterProperty.Items.Add("Security");
                    cbbFilterProperty.Items.Add("Asteroid Belts");
                    cbbFilterProperty.Items.Add("Ice Fields");
                    cbbFilterProperty.Items.Add("Moons");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
                case 6: // Belts
                    cbbFilterProperty.Items.Add("Ore Type");
                    cbbFilterProperty.Items.Add("Ice Type");
                    cbbFilterProperty.SelectedIndex = 0;
                    break;
            }
        }

        private void cbbFilterProperty_SelectedIndexChanged(object sender, EventArgs e)
        {
            cbbFilterCondition.Text = "";
            cbbFilterCondition.Items.Clear();
            cbbFilterValue.Text = "";
            cbbFilterValue.Items.Clear();
            tbFilterValue.Clear();

            cbbFilterCondition.Enabled = false;
            cbbFilterValue.Enabled = false;
            tbFilterValue.Enabled = false;
            btnFilterAdd.Enabled = false;

            switch (cbbFilterGroup.SelectedIndex)
            {
                    /*case 0: // NPC Faction
                    cbbFilterValue.Enabled = true;
                    for (var i = 0; i < EVEData.Instance.FactionList.Count; i++)
                        cbbFilterValue.Items.Add(EVEData.Instance.FactionList[i].Name);
                    cbbFilterValue.SelectedIndex = 0;
                    btnFilterAdd.Enabled = true;
                    break;
                case 1: // NPC Corporation
                    cbbFilterValue.Enabled = true;
                    for (var i = 0; i < EVEData.Instance.FactionList.Count; i++)
                    {
                        var f = EVEData.Instance.FactionList[i];
                        for (var j = 0; j < f.CorpNPCs.Count; j++)
                        {
                            cbbFilterValue.Items.Add(f.CorpNPCs[j].Name);
                        }
                    }
                    cbbFilterValue.SelectedIndex = 0;
                    btnFilterAdd.Enabled = true;
                    break;*/
                case 2: // Agent
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Level
                            tbFilterValue.Enabled = true;
                            cbbFilterCondition.Enabled = true;
                            cbbFilterCondition.Items.Add("Equal");
                            cbbFilterCondition.Items.Add("Equal or higher");
                            cbbFilterCondition.Items.Add("Equal or lower");
                            cbbFilterCondition.Items.Add("Higher");
                            cbbFilterCondition.Items.Add("Lower");
                            cbbFilterCondition.SelectedIndex = 0;
                            break;
                        case 1: // Quality
                            tbFilterValue.Enabled = true;
                            cbbFilterCondition.Enabled = true;
                            cbbFilterCondition.Items.Add("Equal");
                            cbbFilterCondition.Items.Add("Equal or higher");
                            cbbFilterCondition.Items.Add("Equal or lower");
                            cbbFilterCondition.Items.Add("Higher");
                            cbbFilterCondition.Items.Add("Lower");
                            cbbFilterCondition.SelectedIndex = 0;
                            break;
                        case 2: // Locator
                            btnFilterAdd.Enabled = true;
                            break;
                            /*case 3: // Division
                            cbbFilterValue.Enabled = true;
                            for (var i = 0; i < EVEData.Instance.AgentMetaList.AgentDivisions.Count; i++)
                                cbbFilterValue.Items.Add(EVEData.Instance.AgentMetaList.AgentDivisions[i].Name);
                            btnFilterAdd.Enabled = true;
                            break;
                        case 4: // Type
                            cbbFilterValue.Enabled = true;
                            for (var i = 0; i < EVEData.Instance.AgentMetaList.AgentTypes.Count; i++)
                                cbbFilterValue.Items.Add(EVEData.Instance.AgentMetaList.AgentTypes[i].Name);
                            btnFilterAdd.Enabled = true;
                            break;*/
                    }
                    break;
                case 3: // Station
                    btnFilterAdd.Enabled = true;
                    break;
                case 4: // Region
                    cbbFilterValue.Enabled = true;
                    foreach (Region region in StaticStore.Instance.Regions)
                        cbbFilterValue.Items.Add(region.Name);
                    cbbFilterValue.SelectedIndex = 0;
                    btnFilterAdd.Enabled = true;
                    break;
                case 5: // System
                    tbFilterValue.Enabled = true;
                    cbbFilterCondition.Enabled = true;
                    if (cbbFilterProperty.SelectedIndex == 0)
                    {
                        cbbFilterCondition.Items.Add("Higher");
                        cbbFilterCondition.Items.Add("Lower");
                        cbbFilterCondition.SelectedIndex = 0;
                    }
                    else
                    {
                        cbbFilterCondition.Items.Add("Equal");
                        cbbFilterCondition.Items.Add("Equal or higher");
                        cbbFilterCondition.Items.Add("Equal or lower");
                        cbbFilterCondition.Items.Add("Higher");
                        cbbFilterCondition.Items.Add("Lower");
                        cbbFilterCondition.SelectedIndex = 0;
                    }
                    break;
                case 6: // Belts
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Ore Type
                            cbbFilterValue.Enabled = true;
                            cbbFilterValue.Items.Add("Veldspar");
                            cbbFilterValue.Items.Add("Scordite");
                            cbbFilterValue.Items.Add("Pyroxeres");
                            cbbFilterValue.Items.Add("Plagioclase");
                            cbbFilterValue.Items.Add("Jaspet");
                            cbbFilterValue.Items.Add("Gneiss");
                            cbbFilterValue.Items.Add("Kernite");
                            cbbFilterValue.Items.Add("Omber");
                            cbbFilterValue.Items.Add("Spodumain");
                            cbbFilterValue.Items.Add("Hemorphite");
                            cbbFilterValue.Items.Add("Hedbergite");
                            cbbFilterValue.Items.Add("Dark Ochre");
                            cbbFilterValue.Items.Add("Crokite");
                            cbbFilterValue.Items.Add("Bistot");
                            cbbFilterValue.Items.Add("Mercoxit");
                            cbbFilterValue.Items.Add("Arkonor");
                            cbbFilterValue.SelectedIndex = 0;
                            break;
                        case 1: // Ice Type
                            cbbFilterValue.Enabled = true;
                            cbbFilterValue.Items.Add("Blue Ice");
                            cbbFilterValue.Items.Add("Clear Icicle");
                            cbbFilterValue.Items.Add("White Glaze");
                            cbbFilterValue.Items.Add("Glacial Mass");
                            cbbFilterValue.Items.Add("Enriched Clear Icicle");
                            cbbFilterValue.Items.Add("Pristine White Glaze");
                            cbbFilterValue.Items.Add("Smooth Glacial Mass");
                            cbbFilterValue.Items.Add("Thick Blue Ice");
                            cbbFilterValue.Items.Add("Glare Crust");
                            cbbFilterValue.Items.Add("Dark Glitter");
                            cbbFilterValue.Items.Add("Gelidus");
                            cbbFilterValue.Items.Add("Krystallos");
                            cbbFilterValue.SelectedIndex = 0;
                            break;
                    }
                    btnFilterAdd.Enabled = true;
                    break;
            }
        }

        private void btnFilterAdd_Click(object sender, EventArgs e)
        {
            var newFilter = new Filter();

            switch (cbbFilterGroup.SelectedIndex)
            {
                    /*case 0: // NPC Faction
                    newFilter.Type = EVEFilterType.FactionID;
                    newFilter.IntValue = EVEData.Instance.FactionList[cbbFilterValue.SelectedIndex].ID;
                    break;
                case 1: // NPC Corporation
                    newFilter.Type = EVEFilterType.CorporationID;
                    newFilter.IntValue = EVEData.Instance.FactionList.GetCorpNPCByName(cbbFilterValue.Text).ID;
                    break;*/
                case 2: // Agent
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Level
                            newFilter.Type = FilterType.AgentLevel;
                            newFilter.Compare = (FilterCompare) cbbFilterCondition.SelectedIndex;
                            newFilter.IntValue = int.Parse(tbFilterValue.Text);
                            break;
                        case 1: // Quality
                            newFilter.Type = FilterType.AgentQuality;
                            newFilter.Compare = (FilterCompare) cbbFilterCondition.SelectedIndex;
                            newFilter.IntValue = int.Parse(tbFilterValue.Text);
                            break;
                        case 2: // Locator
                            newFilter.Type = FilterType.AgentIsLocator;
                            break;
                            /*case 3: // Division
                            newFilter.Type = EVEFilterType.AgentDivisionID;
                            newFilter.IntValue =
                                EVEData.Instance.AgentMetaList.AgentDivisions[cbbFilterValue.SelectedIndex].ID;
                            break;*/
                            /*case 4: // Type
                            newFilter.Type = EVEFilterType.AgentTypeID;
                            newFilter.IntValue =
                                EVEData.Instance.AgentMetaList.AgentTypes[cbbFilterValue.SelectedIndex].ID;
                            break;*/
                    }
                    break;
                case 3: // Station
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Reprocessing
                            newFilter.Type = FilterType.StationReprocessing;
                            break;
                        case 1: // Manufacturing
                            newFilter.Type = FilterType.StationManufacturing;
                            break;
                        case 2: // Cloning
                            newFilter.Type = FilterType.StationCloning;
                            break;
                    }
                    break;
                case 4: // Region
                    newFilter.Type = FilterType.RegionID;
                    newFilter.IntValue = StaticStore.Instance.Regions[cbbFilterValue.SelectedIndex].ID;
                    break;
                case 5: // System
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Security
                            newFilter.Type = FilterType.SystemSecurity;
                            newFilter.Compare = (FilterCompare) (cbbFilterCondition.SelectedIndex + 3);
                            newFilter.DoubleValue = double.Parse(tbFilterValue.Text);
                            break;
                        case 1: // Asteroid Belt Count
                            newFilter.Type = FilterType.SystemAsteroidBeltCount;
                            newFilter.Compare = (FilterCompare) cbbFilterCondition.SelectedIndex;
                            newFilter.IntValue = int.Parse(tbFilterValue.Text);
                            break;
                        case 2: // Ice Field Belt Count
                            newFilter.Type = FilterType.SystemIceFieldCount;
                            newFilter.Compare = (FilterCompare) cbbFilterCondition.SelectedIndex;
                            newFilter.IntValue = int.Parse(tbFilterValue.Text);
                            break;
                        case 3: // Moon Count
                            newFilter.Type = FilterType.SystemMoonCount;
                            newFilter.Compare = (FilterCompare) cbbFilterCondition.SelectedIndex;
                            newFilter.IntValue = int.Parse(tbFilterValue.Text);
                            break;
                    }
                    break;
                case 6: // Belts
                    switch (cbbFilterProperty.SelectedIndex)
                    {
                        case 0: // Ore Type
                            newFilter.Type = FilterType.BeltOreType;
                            newFilter.IntValue = 1 << cbbFilterValue.SelectedIndex;
                            break;
                        case 1: // Ice Type
                            newFilter.Type = FilterType.BeltIceType;
                            newFilter.IntValue = 1 << cbbFilterValue.SelectedIndex;
                            break;
                    }
                    break;
            }

            mainFilterList.Add(newFilter);
            UpdateFilterList();
        }

        private void tbFilterValue_TextChanged(object sender, EventArgs e)
        {
            if (tbFilterValue.Text == "") return;

            if (cbbFilterGroup.SelectedIndex == 5 && cbbFilterProperty.SelectedIndex == 0)
            {
                try
                {
                    double.Parse(tbFilterValue.Text);
                    tbFilterValue.ForeColor = Color.Black;
                    btnFilterAdd.Enabled = true;
                }
                catch
                {
                    tbFilterValue.ForeColor = Color.Red;
                    btnFilterAdd.Enabled = false;
                }
            }
            else
            {
                try
                {
                    int.Parse(tbFilterValue.Text);
                    tbFilterValue.ForeColor = Color.Black;
                    btnFilterAdd.Enabled = true;
                }
                catch
                {
                    tbFilterValue.ForeColor = Color.Red;
                    btnFilterAdd.Enabled = false;
                }
            }
        }

        private void UpdateFilterList()
        {
            lvFilterList.Items.Clear();

            for (int i = 0; i < mainFilterList.Count; i++)
            {
                Filter f = mainFilterList[i];

                string typeString = "";
                switch (f.Type)
                {
                    case FilterType.FactionID:
                        typeString = "Faction";
                        break;
                    case FilterType.CorporationID:
                        typeString = "Corporation";
                        break;
                    case FilterType.AgentLevel:
                        typeString = "Agent Level";
                        break;
                    case FilterType.AgentQuality:
                        typeString = "Agent Quality";
                        break;
                    case FilterType.AgentIsLocator:
                        typeString = "Agent Locator";
                        break;
                    case FilterType.AgentDivisionID:
                        typeString = "Agent Division";
                        break;
                    case FilterType.AgentTypeID:
                        typeString = "Agent Type";
                        break;
                    case FilterType.StationReprocessing:
                        typeString = "Reprocessing";
                        break;
                    case FilterType.StationManufacturing:
                        typeString = "Manufacturing";
                        break;
                    case FilterType.StationCloning:
                        typeString = "Cloning";
                        break;
                    case FilterType.RegionID:
                        typeString = "Region";
                        break;
                    case FilterType.SystemSecurity:
                        typeString = "Security";
                        break;
                    case FilterType.SystemAsteroidBeltCount:
                        typeString = "Asteroid Belts";
                        break;
                    case FilterType.SystemIceFieldCount:
                        typeString = "Ice Fields";
                        break;
                    case FilterType.SystemMoonCount:
                        typeString = "Moons";
                        break;
                    case FilterType.BeltOreType:
                        typeString = "Ore";
                        break;
                    case FilterType.BeltIceType:
                        typeString = "Ice";
                        break;
                }
                var lvi = new ListViewItem(typeString);

                if (f.Type == FilterType.AgentLevel ||
                    f.Type == FilterType.AgentQuality ||
                    f.Type == FilterType.SystemSecurity ||
                    f.Type == FilterType.SystemAsteroidBeltCount ||
                    f.Type == FilterType.SystemIceFieldCount ||
                    f.Type == FilterType.SystemMoonCount)
                    lvi.SubItems.Add(Filter.GetCompareString(f.Compare));
                else
                    lvi.SubItems.Add("");

                /*if (f.Type == EVEFilterType.FactionID)
                    lvi.SubItems.Add(EVEData.Instance.FactionList.GetByID(f.IntValue).Name);
                else if (f.Type == EVEFilterType.CorporationID)
                    lvi.SubItems.Add(EVEData.Instance.FactionList.GetCorpNPCByID(f.IntValue).Name);
                else if (f.Type == EVEFilterType.AgentDivisionID)
                    lvi.SubItems.Add(EVEData.Instance.AgentMetaList.GetAgentDivisionByID(f.IntValue).Name);
                else if (f.Type == EVEFilterType.AgentTypeID)
                    lvi.SubItems.Add(EVEData.Instance.AgentMetaList.GetAgentTypeByID(f.IntValue).Name);*/
                //else 
                if (f.Type == FilterType.StationReprocessing ||
                    f.Type == FilterType.StationManufacturing ||
                    f.Type == FilterType.StationCloning)
                    lvi.SubItems.Add("");
                else if (f.Type == FilterType.RegionID)
                    lvi.SubItems.Add(StaticStore.Instance.Regions[f.IntValue].Name);
                else if (f.Type == FilterType.SystemSecurity)
                    lvi.SubItems.Add(f.DoubleValue.ToString("F"));
                else if (f.Type == FilterType.BeltOreType)
                    lvi.SubItems.Add(((Minerals.OreTypes) (f.IntValue)).ToString());
                else if (f.Type == FilterType.BeltIceType)
                    lvi.SubItems.Add(((Minerals.IceTypes) (f.IntValue)).ToString());
                else
                    lvi.SubItems.Add(f.IntValue.ToString());

                lvFilterList.Items.Add(lvi);
            }
            mapView.UnfilteredSystems = new Dictionary<int, SolarSystem>();
            foreach (KeyValuePair<int, SolarSystem> system in StaticStore.Systems)
            {
                if (mainFilterList.TestSystem(system.Value))
                {
                    mapView.UnfilteredSystems.Add(system.Key, system.Value);
                }
            }
            if (mapView.UnfilteredSystems.Count > 0)
            {
                mapView.FilterType = MapViewport.SystemFilterType.UserFilter;
                mapView.FilterSystems = true;
            }
            else
            {
                mapView.FilterType = MapViewport.SystemFilterType.None;
                mapView.FilterSystems = false;
            }
            mapView.Invalidate();
        }

        private void btnFilterRemove_Click(object sender, EventArgs e)
        {
            if (lvFilterList.SelectedIndices.Count != 1) return;

            mainFilterList.RemoveAt(lvFilterList.SelectedIndices[0]);
            UpdateFilterList();
        }

        private void btnFilterClear_Click(object sender, EventArgs e)
        {
            mainFilterList.Clear();
            UpdateFilterList();
            mapView.UnfilteredSystems = null;
            mapView.FilterSystems = false;
        }

        private void cbbFilterScope_SelectedIndexChanged(object sender, EventArgs e)
        {
            mainFilterList.Scope = (FilterScope) cbbFilterScope.SelectedIndex;
            mapView.Invalidate();
        }

        #endregion
    }
}