using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Renderer.Text;
using libEveMapNG.Utility;

namespace EveMapControls
{
    public partial class SystemSelectBox : ComboBox
    {
        /// <summary>
        ///   API constant and Function Declerations
        /// </summary>
        private const uint WM_SETCURSOR = 32;

        private readonly EventHandler delayedDrawer;

        public SystemSelectBox()
        {
            InitializeComponent();
            DrawMode = DrawMode.OwnerDrawFixed;
            DisplayMember = "Name";
            delayedDrawer = DelayCallback;
        }

        public SolarSystem SelectedSystem
        {
            get { return (SolarSystem) SelectedItem; }
            set
            {
                if (value != null)
                {
                    Items.Clear();
                    Items.Add(value);
                    SelectedValue = value;
                    SelectedIndex = 0;
                }
            }
        }

        public bool LimitToJumpDestinations { get; set; }

        [DllImport("user32.dll", EntryPoint = "SendMessageA")]
        private static extern void SendMessage(IntPtr hWnd, uint Msg, long wParam, long lParam);

        protected override void OnDrawItem(DrawItemEventArgs e)
        {
            e.DrawBackground();
            var systemMarkerBrush = new SolidBrush(Color.Black);
            var foreBrush = new SolidBrush(ForeColor);
            var markerTexts = new String[3];

            if (e.Index >= 0 && Items[e.Index] is SolarSystem)
            {
                bool isHighlighted = ((e.State & DrawItemState.Selected) == DrawItemState.Selected);

                var drawItem = (SolarSystem) Items[e.Index];

                Color secColor = SolarSystemColorSchemes.SecurityScheme.GetSystemColor(drawItem);

                //space between icon/system and system/region
                const int spacing = 2;
                //marker
                int markerX = e.Bounds.X;
                int markerY = e.Bounds.Y;
                if (drawItem.Stations == null || drawItem.Stations.Count == 0)
                    markerTexts[0] = "(S";
                else
                    markerTexts[0] = "(-";

                int systemJbCount =
                    (from jb in DynamicStore.Instance.JumpBridges
                     where jb.From.ID == drawItem.ID || jb.To.ID == drawItem.ID
                     select jb).Count();

                if (systemJbCount > 0)
                    markerTexts[1] = "J";
                else
                    markerTexts[1] = "-";

                int systemBeconCount =
                    (from b in DynamicStore.Instance.CynoBeacons
                     where b.Value.System == drawItem
                     select b).Count();

                if (systemBeconCount > 0)
                    markerTexts[2] = "B)";
                else
                    markerTexts[2] = "-)";

                string markerText = String.Join("", markerTexts);

                int markerLength = Size.Round(e.Graphics.MeasureString(markerText, Font)).Width;
                //system
                string systemString = drawItem.Name;
                int systemX = e.Bounds.X + spacing + markerLength;
                int systemY = e.Bounds.Y;
                int systemNameOffset = Size.Round(e.Graphics.MeasureString(drawItem.Name, Font)).Width;

                //region
                string regionString = string.Format("[{0}]",
                                                    libEveMapNG.Data.Static.Region.GetByID(drawItem.RegionID).Name);
                int regionX = e.Bounds.X + spacing + markerLength + spacing + systemNameOffset;
                int regionY = e.Bounds.Y;

                e.Graphics.SmoothingMode = SmoothingMode.HighSpeed;

                SolidBrush regionNameBrush = (isHighlighted) ? foreBrush : new SolidBrush(secColor);
                e.Graphics.DrawString(markerText, Font, systemMarkerBrush, markerX, markerY,
                                      StringFormat.GenericDefault);
                e.Graphics.DrawString(systemString, Font, foreBrush, systemX, systemY,
                                      StringFormat.GenericDefault);
                e.Graphics.DrawString(regionString, Font, regionNameBrush, regionX, regionY,
                                      StringFormat.GenericDefault);

                regionNameBrush.Dispose();
            }

            foreBrush.Dispose();
            systemMarkerBrush.Dispose();

            e.DrawFocusRectangle();
            base.OnDrawItem(e);
        }

        protected override void OnDropDown(EventArgs e)
        {
            base.OnDropDown(e);

            //HACK!
            //this will force the mouse to show up when the drop down is done
            SendMessage(Handle, WM_SETCURSOR, 0, 0);
            //the  mouse will hide when the drop down is shown via code
            //this is a known issue witht he combo box.
        }

        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            base.OnKeyPress(e);
            //dont do anything, it will all be handled in the key up

            return;
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            try
            {
                base.OnKeyDown(e);
            }
            catch
            {
                return;
            }
            //dont do anything, it will all be handled in the key up

            return;
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            /*if (e.KeyCode == Keys.Back && SelectionStart > 0)
            {
                SelectionStart--;
            }*/

            //hide it if escape or enter
            if (e.KeyCode == Keys.Escape || e.KeyCode == Keys.Enter)
                DroppedDown = false;
                //The - on the numpad is a '-' and the minus on the main keyboard is a ''
            else if ((Char.IsLetterOrDigit((char) e.KeyCode) || (char) e.KeyCode == '-' || (char) e.KeyCode == '' ||
                      e.KeyCode == Keys.Back) && !e.Control && !e.Alt)
            {
                //delay.Change(33, System.Threading.Timeout.Infinite);
                DelayCallback(null, null);
            }
            else
            {
                //if not a printable character then ignore and use base class
                base.OnKeyUp(e);
                return;
            }
        }

        protected void DelayCallbackWrapper(Object stateInfo)
        {
            Invoke(delayedDrawer);
        }

        protected void DelayCallback(Object sender, EventArgs e)
        {
            //DroppedDown = false;


            int selectionStart = SelectionStart;
            string currentText = Text.Substring(0, selectionStart);

            if (Items.Count > 0)
                Items.Clear();

            List<SolarSystem> results = GuiTools.SearchForSystem(currentText, LimitToJumpDestinations);

            foreach (SolarSystem system in results)
            {
                Items.Add(system);
            }
            //autocompelte helper
            if (Items.Count > 0)
            {
                SelectedItem = Items[0];
                Text = ((SolarSystem) Items[0]).Name;
            }
            else
                Text = currentText;
            DroppedDown = true;
            SelectionStart = selectionStart;

            return;
        }
    }
}