#region

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Text;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using EveMapControls.Viewport;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.EveAPI.Data;
using libEveMapNG.Filtering;
using libEveMapNG.Renderer.Text;
using libEveMapNG.Types;
using libEveMapNG.Utility;
using Region = libEveMapNG.Data.Static.Region;

#endregion

namespace EveMapControls
{
    public partial class MapViewport : IMapViewport
    {
        private const float DEFAULT_CONST_FONT_SIZE = 32;
        private const float DEFAULT_REGION_FONT_SIZE = 92;
        private const float DEFAULT_SYSTEM_DETAIL_SIZE = 8;
        private const float DEFAULT_SYSTEM_FONT_SIZE = 8;

        private static NumberFormatInfo _ni;

        private Point _adjust;
        private Font _constFont;
        private bool _flatDataChanged = true;
        private bool _isInitialized;
        private int _labelAlpha = 0x40;
        private int _lineAlpha = 0xFF;
        private float _lineThickness;
        private Point _mapDrag;
        private bool _mapDragging;
        private Point _mapDragOffset;
        private Size _mapEffectiveSize;
        private float _mapScale;

        private Font _regionFont;
        private bool _scrollEnabled = true;

        private Font _systemBoldFont, _systemDetailFont;
        private Font _systemFont;
        private bool _zoomEnabled = true;

        #region Routing and filtering

        private SolarSystem _hoverSystem;

        private SolarSystem _selectedSystem;

        #endregion

        #region Minimap stuff

        private MiniMapAlignment _miniMapAlign = MiniMapAlignment.Top | MiniMapAlignment.Left;
        private Bitmap _miniMapBitmap;
        private bool miniMapDragging;
        private Point miniMapDragOffset;
        private bool miniMapHandlesVisible;
        private Size miniMapMargin = Size.Empty;
        private bool miniMapOutlineDragging;
        private Point miniMapOutlineDragOffset;
        private Rectangle miniMapOutlineRect = Rectangle.Empty;
        private Rectangle miniMapRect = new Rectangle(10, 10, 200, 225);
        private SizeF miniMapScale = Size.Empty;

        [Flags]
        private enum MiniMapAlignment
        {
            None = 0x0000,
            Top = 0x0001,
            Left = 0x0010,
            Bottom = 0x0100,
            Right = 0x1000
        }

        #endregion

        #region Public properties

        public int PointX
        {
            get { return _adjust.X; }
            set { _adjust.X = value; }
        }

        public int PointY
        {
            get { return _adjust.Y; }
            set { _adjust.Y = value; }
        }

        public String SelectedSystemName
        {
            set
            {
                SolarSystem solar = SolarSystem.GetByName(value);
                if (solar != null)
                    SelectedSystem = solar;
            }
            get
            {
                if (_selectedSystem != null)
                    return _selectedSystem.Name;
                return String.Empty;
            }
        }


        public SolarSystem SelectedSystem
        {
            get { return _selectedSystem; }
            set
            {
                if (value == null)
                    return;

                _selectedSystem = value;
                OnSystemSelected();

                CenterOnPoint(_selectedSystem.OrganicCoordinates);

                Application.DoEvents();
            }
        }

        public SolarSystem HoverSystem
        {
            get { return _hoverSystem; }
        }

        public float MapScale
        {
            get { return _mapScale; }
            set
            {
                float oldScale = _mapScale;
                _mapScale = value;
                AdjustGraphicsToScale(oldScale, true);
            }
        }

        public bool ScrollEnabled
        {
            get { return _scrollEnabled; }
            set
            {
                _scrollEnabled = value;
                AdjustScrollBars();
                Invalidate();
            }
        }

        public bool ZoomEnabled
        {
            get { return _zoomEnabled; }
            set { _zoomEnabled = value; }
        }

        public ContextMenuStrip SystemContextMenu { get; set; }

        #endregion

        #region Events

        // Event for when a system has been selected

        #region Delegates

        public delegate void SystemSelectedHandler();

        #endregion

        public event SystemSelectedHandler SystemSelected;

        protected virtual void OnSystemSelected()
        {
            if (SystemSelected != null)
                SystemSelected();
        }

        #endregion

        public MapViewport()
        {
            InitializeComponent();

            sbHorizontal.Visible = sbVertical.Visible = cornerPanel.Visible = _scrollEnabled;

            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            ResizeRedraw = false;

            // Force string/double conversions to use "." as decimal point
            // InstalledUICulture is read-only so it needs to be copied and changed
            CultureInfo ci =
                CultureInfo.InstalledUICulture;
            _ni = (NumberFormatInfo)ci.NumberFormat.Clone();
            _ni.NumberDecimalSeparator = ".";
        }

        #region IMapViewport Members

        public void InitializeMapSettings()
        {
            _isInitialized = true;
            FillTerritoryParameters();
            _mapScale = 1.0f;
            _mapEffectiveSize = ClientSize;
            CalculateMinimapAlignment();
            AdjustGraphicsToScale(1.0f, false);
        }

        #endregion

        protected override void OnPaint(PaintEventArgs e)
        {
            Stopwatch myWatch = new Stopwatch();
            myWatch.Start();
            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;
            e.Graphics.TextRenderingHint = TextRenderingHint.SystemDefault;

            if (!_isInitialized) return;

            e.Graphics.Clear(UserSettings.Instance.ActiveColorScheme.BackgroundColor);
            FillTerritoryParameters();

            // we extend it slightly to avoid clipping off AAed edges of things, for example gate lines
            e.ClipRectangle.Inflate(2, 2);

            DrawSystems(e.Graphics, e.ClipRectangle);
            DrawConnections(e.Graphics, e.ClipRectangle);
            PaintMinimap(e.Graphics);
            myWatch.Stop();
            Debug.WriteLine(myWatch.Elapsed);
            myWatch.Reset();
        }

        public void RefreshMinimap()
        {
            MiniMapFillBitmap();
        }

        private void PaintMinimap(Graphics g)
        {
            if (UserSettings.Instance.ActiveViewportStyle.ShowMiniMap)
            {
                if (_miniMapBitmap == null)
                    MiniMapFillBitmap();

                using (
                    Pen shadowPen =
                        new Pen(Color.FromArgb(0x40, UserSettings.Instance.ActiveColorScheme.MinimapBorderColor)),
                        shadowPen2 =
                            new Pen(Color.FromArgb(0x80, UserSettings.Instance.ActiveColorScheme.MinimapBorderColor)))
                {
                    g.DrawRectangle(shadowPen, miniMapRect.Left + 2, miniMapRect.Top + 2, miniMapRect.Width - 1,
                                    miniMapRect.Height - 1);

                    g.DrawRectangle(shadowPen2, miniMapRect.Left + 1, miniMapRect.Top + 1, miniMapRect.Width - 1,
                                    miniMapRect.Height - 1);
                }

                if (miniMapHandlesVisible)
                    MiniMapDrawResizeDisplay(g);
                else
                {
                    g.DrawImageUnscaled(_miniMapBitmap, miniMapRect);
                    MiniMapDrawOutline(g);
                }
            }
        }

        private void FillTerritoryParameters()
        {
            if (!_isInitialized) return;

            if (_flatDataChanged)
            {
                GetGlobalFlatArea();
                _flatDataChanged = false;
                AdjustScrollBars();
            }

            _adjust.X = sbHorizontal.Value + _globalFlatArea.Left -
                        UserSettings.Instance.ActiveViewportStyle.Margin.Width;
            _adjust.Y = sbVertical.Value + _globalFlatArea.Top - UserSettings.Instance.ActiveViewportStyle.Margin.Height;
        }

        private void AdjustScrollBars()
        {
            if (!_isInitialized) return;

            sbHorizontal.Visible = sbVertical.Visible = cornerPanel.Visible = _scrollEnabled;

            sbHorizontal.Minimum = 0;
            sbHorizontal.Maximum = _globalFlatArea.Width + 2 * UserSettings.Instance.ActiveViewportStyle.Margin.Width;

            sbVertical.Minimum = 0;
            sbVertical.Maximum = _globalFlatArea.Height + 2 * UserSettings.Instance.ActiveViewportStyle.Margin.Height;

            if (_scrollEnabled)
            {
                _mapEffectiveSize = Size.Subtract(ClientSize, new Size(sbVertical.Width, sbHorizontal.Height));

                if (_globalFlatArea.Width + UserSettings.Instance.ActiveViewportStyle.Margin.Width * 2 <
                    _mapEffectiveSize.Width / _mapScale)
                {
                    sbHorizontal.Enabled = false;
                    sbHorizontal.Value = 0;
                }
                else
                {
                    sbHorizontal.Enabled = true;
                    sbHorizontal.LargeChange = (int)(_mapEffectiveSize.Width / _mapScale);
                    sbHorizontal.SmallChange = (int)(_mapEffectiveSize.Width / 5 / _mapScale);
                }

                if (_globalFlatArea.Height + UserSettings.Instance.ActiveViewportStyle.Margin.Height * 2 <
                    _mapEffectiveSize.Height / _mapScale)
                {
                    sbVertical.Enabled = false;
                    sbVertical.Value = 0;
                }
                else
                {
                    //Fix the Large Change Bug????
                    if (_mapEffectiveSize.Height < 0)
                        _mapEffectiveSize.Height = 500;

                    sbVertical.Enabled = true;
                    sbVertical.LargeChange = (int)(_mapEffectiveSize.Height / _mapScale);
                    sbVertical.SmallChange = (int)(_mapEffectiveSize.Height / 5 / _mapScale);
                }
            }
            else _mapEffectiveSize = ClientSize;
        }

        private void AdjustGraphicsToScale(float oldScale, bool recenter)
        {
            if (!_isInitialized) return;

            float systemFontSize = DEFAULT_SYSTEM_FONT_SIZE,
                  systemDetailFontSize = DEFAULT_SYSTEM_DETAIL_SIZE,
                  systemBoldFontSize = DEFAULT_SYSTEM_FONT_SIZE,
                  regionFontSize = DEFAULT_REGION_FONT_SIZE,
                  constFontSize = DEFAULT_CONST_FONT_SIZE;

            _lineThickness = UserSettings.Instance.ActiveViewportStyle.LineThickness *
                             ((_mapScale < 0.8f) ? 0.8f : _mapScale);

            _lineAlpha = 0xFF;
            _labelAlpha = 0x40;

            if (_mapScale < 0.5f) // dim out gates, display labels prominently
            {
                _lineAlpha = 0xFF - (int)((0.5f - _mapScale) * 25 * 16);
                _labelAlpha = 0x40 + (int)((0.5f - _mapScale) * 25 * 16);
            }

            #region Font scaling

            if (_mapScale < 0.50)
            {
                // details are hidden at these scales
                systemFontSize = 7;
                systemBoldFontSize = 7;
                systemDetailFontSize = 7;
                regionFontSize = DEFAULT_REGION_FONT_SIZE * (_mapScale + 0.08f);
                constFontSize = DEFAULT_CONST_FONT_SIZE * (_mapScale + 0.08f);
            }
            if (_mapScale >= 0.50 && _mapScale < 0.80)
            {
                systemFontSize = 7;
                systemBoldFontSize = 7;
                systemDetailFontSize = 7;
                regionFontSize = DEFAULT_REGION_FONT_SIZE * (_mapScale + 0.08f);
                constFontSize = DEFAULT_CONST_FONT_SIZE * (_mapScale + 0.08f);
            }
            else if (_mapScale >= 0.80 && _mapScale < 0.90)
            {
                systemFontSize = 8;
                systemBoldFontSize = 8;
                systemDetailFontSize = 7;
                regionFontSize = DEFAULT_REGION_FONT_SIZE * (_mapScale + 0.08f);
                constFontSize = DEFAULT_CONST_FONT_SIZE * (_mapScale + 0.08f);
            }
            else if (_mapScale >= 0.90 && _mapScale < 1.10)
            {
                systemFontSize = 9;
                systemBoldFontSize = 9;
                systemDetailFontSize = 8;
            }
            else if (_mapScale >= 1.10 && _mapScale < 1.20)
            {
                systemFontSize = 10;
                systemBoldFontSize = 10;
                systemDetailFontSize = 9;
            }
            else if (_mapScale >= 1.20 && _mapScale <= 1.50)
            {
                systemFontSize = 11;
                systemBoldFontSize = 11;
                systemDetailFontSize = 10;
            }

            _systemFont = new Font(UserSettings.Instance.ActiveViewportStyle.SystemLabelFontFamily, systemFontSize);
            _systemBoldFont = new Font(UserSettings.Instance.ActiveViewportStyle.SystemLabelFontFamily,
                                       systemBoldFontSize,
                                       FontStyle.Bold);
            _systemDetailFont = new Font(UserSettings.Instance.ActiveViewportStyle.DetailTextFontFamily,
                                         systemDetailFontSize);
            _regionFont = new Font(UserSettings.Instance.ActiveViewportStyle.MapLabelFontFamily, regionFontSize);
            _constFont = new Font(UserSettings.Instance.ActiveViewportStyle.MapLabelFontFamily, constFontSize);

            #endregion

            AdjustScrollBars();

            if (recenter)
            {
                PointF oldCenter =
                    Utility.UnscalePoint(new PointF(_mapEffectiveSize.Width / 2f, _mapEffectiveSize.Height / 2f), _adjust,
                                         oldScale);

                // this is to adjust the drag offset if they zoom while dragging, it's not perfectly accurate yet and i'm not sure why
                // but it's better than nothing

                if (miniMapOutlineDragging)
                {
                    //FastMap_MouseDown(null, new MouseEventArgs(MouseButtons.Left, 1, Cursor.Position.X, Cursor.Position.Y, 0));
                    miniMapOutlineDragOffset.X =
                        (int)
                        (((_mapEffectiveSize.Width / (sbHorizontal.Maximum * _mapScale) * miniMapRect.Width - 2) -
                          (_mapEffectiveSize.Width / (sbHorizontal.Maximum * oldScale) * miniMapRect.Width - 2)) / 2 +
                         miniMapOutlineDragOffset.X);
                    miniMapOutlineDragOffset.Y =
                        (int)
                        (((_mapEffectiveSize.Height / (sbVertical.Maximum * _mapScale) * miniMapRect.Height - 2) -
                          (_mapEffectiveSize.Height / (sbVertical.Maximum * oldScale) * miniMapRect.Height - 2)) / 2 +
                         miniMapOutlineDragOffset.Y);
                }

                CenterOnPoint(oldCenter);
            }
        }

        public void ViewOptionsMapVisualsChanged(bool refreshMinimap, bool refreshScaledElements)
        {
            if (refreshMinimap) MiniMapFillBitmap();
            if (refreshScaledElements) AdjustGraphicsToScale(_mapScale, false);
            Refresh();
        }

        private void UserScroll(object sender, ScrollEventArgs e)
        {
            Invalidate();
        }

        private void MapViewport_Layout(object sender, LayoutEventArgs e)
        {
            if (!_isInitialized) return;

            AdjustScrollBars();

            if ((_miniMapAlign & MiniMapAlignment.Top) == MiniMapAlignment.Top)
                miniMapRect.Y = miniMapMargin.Height;
            else if ((_miniMapAlign & MiniMapAlignment.Bottom) == MiniMapAlignment.Bottom)
                miniMapRect.Y = _mapEffectiveSize.Height - miniMapMargin.Height - miniMapRect.Height;

            if ((_miniMapAlign & MiniMapAlignment.Left) == MiniMapAlignment.Left)
                miniMapRect.X = miniMapMargin.Width;
            else if ((_miniMapAlign & MiniMapAlignment.Right) == MiniMapAlignment.Right)
                miniMapRect.X = _mapEffectiveSize.Width - miniMapMargin.Width - miniMapRect.Width;

            PlaceMinimapInBounds(false);

            Invalidate(Rectangle.FromLTRB(miniMapRect.Left - 10, miniMapRect.Top - 10,
                                          miniMapRect.Right + 10, miniMapRect.Bottom + 10));
        }

        #region Utility geometric and scrolling functions

        // converting screen to/from SolarSystem.FlatX/Y coordinates
        // Scale: from flat xy to screen xy
        // Unscale: from screen xy to flat xy
        // note: adjustX/Y are set by AdjustScrollBars()

        private Rectangle _globalFlatArea;

        public void CenterOnPoint(PointF point)
        {
            float newScrollX = point.X - _globalFlatArea.Left + UserSettings.Instance.ActiveViewportStyle.Margin.Width -
                               _mapEffectiveSize.Width / 2 / _mapScale;
            float newScrollY = point.Y - _globalFlatArea.Top + UserSettings.Instance.ActiveViewportStyle.Margin.Height -
                               _mapEffectiveSize.Height / 2 / _mapScale;

            ScrollToPosition(new Point((int)newScrollX, (int)newScrollY));
        }

        private void ScrollToPosition(PointF scrollValues)
        {
            Point safeScroll = SnapToScrollBoundaries(scrollValues);
            sbHorizontal.Value = safeScroll.X;
            sbVertical.Value = safeScroll.Y;

            Invalidate();
        }

        private Point SnapToScrollBoundaries(PointF newScrollPos)
        {
            if (newScrollPos.X > sbHorizontal.Maximum - _mapEffectiveSize.Width / _mapScale)
                newScrollPos.X = (int)(sbHorizontal.Maximum - _mapEffectiveSize.Width / _mapScale);
            if (newScrollPos.Y > sbVertical.Maximum - _mapEffectiveSize.Height / _mapScale)
                newScrollPos.Y = (int)(sbVertical.Maximum - _mapEffectiveSize.Height / _mapScale);

            if (newScrollPos.X < 0) newScrollPos.X = 0;
            if (newScrollPos.Y < 0) newScrollPos.Y = 0;

            return Point.Round(newScrollPos);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="resizeToFit">
        /// if resizeToFit is true, if the minimap is out of bounds, it will "truncate" the extra part off
        /// the width.  if it's false, instead, it will keep the same size and slide the minimap back
        /// until it's on the map edge.
        /// </param>
        /// <remarks>
        /// resizeToFit does not work because PaintMinimap uses Graphics.DrawImageUnscaled to draw the minimap.
        /// </remarks>
        private void PlaceMinimapInBounds(bool resizeToFit)
        {
            if (miniMapRect.X + miniMapRect.Width >= _mapEffectiveSize.Width - 1)
            {
                if (resizeToFit) miniMapRect.Width = _mapEffectiveSize.Width - miniMapRect.X - 1;
                else miniMapRect.X = _mapEffectiveSize.Width - miniMapRect.Width - 1;
            }
            if (miniMapRect.Y + miniMapRect.Height >= _mapEffectiveSize.Height - 1)
            {
                if (resizeToFit) miniMapRect.Height = _mapEffectiveSize.Height - miniMapRect.Y - 1;
                else miniMapRect.Y = _mapEffectiveSize.Height - miniMapRect.Height - 1;
            }
            if (miniMapRect.X < 0)
            {
                if (resizeToFit) miniMapRect.Width = miniMapRect.Right;
                miniMapRect.X = 0;
            }
            if (miniMapRect.Y < 0)
            {
                if (resizeToFit) miniMapRect.Height = miniMapRect.Bottom;
                miniMapRect.Y = 0;
            }
        }

        private void CalculateMinimapAlignment()
        {
            _miniMapAlign = MiniMapAlignment.None;

            if (miniMapRect.Top <= _mapEffectiveSize.Height - miniMapRect.Bottom)
            {
                _miniMapAlign |= MiniMapAlignment.Top;
                miniMapMargin.Height = miniMapRect.Top;
            }
            else
            {
                _miniMapAlign |= MiniMapAlignment.Bottom;
                miniMapMargin.Height = _mapEffectiveSize.Height - miniMapRect.Bottom;
            }

            if (miniMapRect.Left <= _mapEffectiveSize.Width - miniMapRect.Right)
            {
                _miniMapAlign |= MiniMapAlignment.Left;
                miniMapMargin.Width = miniMapRect.Left;
            }
            else
            {
                _miniMapAlign |= MiniMapAlignment.Right;
                miniMapMargin.Width = _mapEffectiveSize.Width - miniMapRect.Right;
            }
        }

        private void GetGlobalFlatArea()
        {
            Point flatMin = new Point(int.MaxValue, int.MaxValue);
            Point flatMax = new Point(int.MinValue, int.MinValue);

            foreach (Region curRegion in StaticStore.Instance.Regions)
            {
                if (curRegion.Bounds.Left < flatMin.X) flatMin.X = curRegion.Bounds.Left;
                if (curRegion.Bounds.Right > flatMax.X) flatMax.X = curRegion.Bounds.Right;
                if (curRegion.Bounds.Top < flatMin.Y) flatMin.Y = curRegion.Bounds.Top;
                if (curRegion.Bounds.Bottom > flatMax.Y) flatMax.Y = curRegion.Bounds.Bottom;
            }

            _globalFlatArea = Rectangle.FromLTRB(flatMin.X, flatMin.Y, flatMax.X, flatMax.Y);
        }

        private SolarSystem FindSystemOnScreen(PointF location, double radius)
        {
            foreach (var pair in StaticStore.Systems)
            {
                if (IsSystemSelectable(pair.Value))
                {
                    PointF sysLoc = Utility.ScalePoint(pair.Value.OrganicCoordinates, _adjust, _mapScale);
                    float xDiff = sysLoc.X - location.X;
                    float yDiff = sysLoc.Y - location.Y;
                    float curRadius = xDiff * xDiff + yDiff * yDiff;
                    if (curRadius < radius * radius) return pair.Value;
                }
            }

            return null;
        }

        private bool IsSystemSelectable(SolarSystem system)
        {
            if (_mapScale < 0.50f)
                return DynamicStore.Instance.CynoBeacons.ContainsKey(system);
            return true;
        }

        #endregion

        #region Input event handlers

        private RelativePosition GetMouseMinimapPosition(Point mouseLocation)
        {
            const byte borderMargin = 7;

            int l1 = miniMapRect.Left - borderMargin, l2 = miniMapRect.Left + borderMargin;
            int t1 = miniMapRect.Top - borderMargin, t2 = miniMapRect.Top + borderMargin;
            int r1 = miniMapRect.Right - borderMargin, r2 = miniMapRect.Right + borderMargin;
            int b1 = miniMapRect.Bottom - borderMargin, b2 = miniMapRect.Bottom + borderMargin;

            if (Rectangle.FromLTRB(l2, t2, r1, b1).Contains(mouseLocation)) return RelativePosition.Center;
            if (Rectangle.FromLTRB(l1, t2, l2, b1).Contains(mouseLocation)) return RelativePosition.CenterLeft;
            if (Rectangle.FromLTRB(r1, t2, r2, b1).Contains(mouseLocation)) return RelativePosition.CenterRight;
            if (Rectangle.FromLTRB(l2, t1, r1, t2).Contains(mouseLocation)) return RelativePosition.Top;
            if (Rectangle.FromLTRB(l1, t1, l2, t2).Contains(mouseLocation)) return RelativePosition.TopLeft;
            if (Rectangle.FromLTRB(r1, t1, r2, t2).Contains(mouseLocation)) return RelativePosition.TopRight;
            if (Rectangle.FromLTRB(l2, b1, r1, b2).Contains(mouseLocation)) return RelativePosition.Bottom;
            if (Rectangle.FromLTRB(l1, b1, l2, b2).Contains(mouseLocation)) return RelativePosition.BottomLeft;
            if (Rectangle.FromLTRB(r1, b1, r2, b2).Contains(mouseLocation)) return RelativePosition.BottomRight;

            return RelativePosition.None;
        }

        private static Cursor GetProperCursor(RelativePosition position)
        {
            switch (position)
            {
                case RelativePosition.Center:
                    return Cursors.SizeAll;
                case RelativePosition.Bottom:
                case RelativePosition.Top:
                    return Cursors.SizeNS;
                case RelativePosition.CenterLeft:
                case RelativePosition.CenterRight:
                    return Cursors.SizeWE;
                case RelativePosition.TopLeft:
                case RelativePosition.BottomRight:
                    return Cursors.SizeNWSE;
                case RelativePosition.BottomLeft:
                case RelativePosition.TopRight:
                    return Cursors.SizeNESW;
                default:
                    return Cursors.Default;
            }
        }

        private void OnMouseDown(object sender, MouseEventArgs e)
        {
            if (miniMapHandlesVisible)
            {
                RelativePosition mmPos = GetMouseMinimapPosition(e.Location);
                if (mmPos == RelativePosition.Center)
                {
                    miniMapDragging = true;

                    miniMapDragOffset.X = e.Location.X - miniMapRect.Left;
                    miniMapDragOffset.Y = e.Location.Y - miniMapRect.Top;
                }
            }
            else if (UserSettings.Instance.ActiveViewportStyle.ShowMiniMap && miniMapRect.Contains(e.Location))
            {
                if (e.Button == MouseButtons.Left)
                {
                    miniMapOutlineDragging = true;

                    if (miniMapOutlineRect.Contains(e.Location))
                    {
                        miniMapOutlineDragOffset.X = e.Location.X - miniMapOutlineRect.Left;
                        miniMapOutlineDragOffset.Y = e.Location.Y - miniMapOutlineRect.Top;
                    }
                    else
                    {
                        miniMapOutlineDragOffset.X = miniMapOutlineRect.Width / 2;
                        miniMapOutlineDragOffset.Y = miniMapOutlineRect.Height / 2;

                        Point newScroll = new Point(
                            (int)((e.Location.X - miniMapRect.Left - miniMapOutlineDragOffset.X) / miniMapScale.Width),
                            (int)((e.Location.Y - miniMapRect.Top - miniMapOutlineDragOffset.Y) / miniMapScale.Height));

                        ScrollToPosition(newScroll);
                    }
                }
                else if (e.Button == MouseButtons.Right)
                {
                    miniMapDragging = true;

                    miniMapDragOffset.X = e.Location.X - miniMapRect.Left;
                    miniMapDragOffset.Y = e.Location.Y - miniMapRect.Top;
                }
            }
            else
            {
                if (e.Button == MouseButtons.Left && _scrollEnabled)
                {
                    // rightclick: drag map around
                    _mapDragging = true;
                    _mapDrag = e.Location;
                    _mapDragOffset.X = sbHorizontal.Value;
                    _mapDragOffset.Y = sbVertical.Value;
                }
            }
        }

        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            if (miniMapOutlineDragging || _mapDragging || miniMapOutlineRect.Contains(e.Location))
                Cursor = Cursors.Hand;
            else if (miniMapHandlesVisible)
                Cursor = GetProperCursor(GetMouseMinimapPosition(e.Location));
            else
                Cursor = Cursors.Default;


            if (miniMapOutlineDragging)
            {
                Cursor = Cursors.Hand;

                Point newScroll = new Point(
                    (int)((e.Location.X - miniMapRect.Left - miniMapOutlineDragOffset.X) / miniMapScale.Width),
                    (int)((e.Location.Y - miniMapRect.Top - miniMapOutlineDragOffset.Y) / miniMapScale.Height));

                ScrollToPosition(newScroll);
            }
            else if (_mapDragging)
            {
                Cursor = Cursors.Hand;

                Point newScroll = new Point(
                    (int)(2 * (_mapDrag.X - e.X) / _mapScale + _mapDragOffset.X),
                    (int)(2 * (_mapDrag.Y - e.Y) / _mapScale + _mapDragOffset.Y));

                ScrollToPosition(newScroll);
            }
            else if (miniMapDragging)
            {
                Rectangle oldMiniMapRect = miniMapRect;

                miniMapRect.X = e.Location.X - miniMapDragOffset.X;
                miniMapRect.Y = e.Location.Y - miniMapDragOffset.Y;

                PlaceMinimapInBounds(false);

                Rectangle redrawRect = Rectangle.Union(miniMapRect, oldMiniMapRect);
                redrawRect.Inflate(2, 2);

                Invalidate(redrawRect);
            }
        }

        private void OnMouseUp(object sender, MouseEventArgs e)
        {
            Cursor = Cursors.Default;

            if (!(UserSettings.Instance.ActiveViewportStyle.ShowMiniMap && miniMapRect.Contains(e.Location)))
            {
                double radius = UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize *
                                ((_mapScale < 0.80f) ? 0.80f : _mapScale);
                SolarSystem selSystem = FindSystemOnScreen(e.Location, radius);

                if (selSystem != null)
                {
                    if (e.Button == MouseButtons.Left)
                    {
                        _selectedSystem = selSystem;
                        OnSystemSelected();
                        Invalidate();
                    }
                    else if (e.Button == MouseButtons.Right && SystemContextMenu != null)
                    {
                        _hoverSystem = selSystem;
                        SystemContextMenu.Show(this, e.Location);
                    }
                }
            }

            if (miniMapOutlineDragging)
            {
                Point newScroll = new Point(
                    (int)((e.Location.X - miniMapRect.Left - miniMapOutlineDragOffset.X) / miniMapScale.Width),
                    (int)((e.Location.Y - miniMapRect.Top - miniMapOutlineDragOffset.Y) / miniMapScale.Height));

                ScrollToPosition(newScroll);
                miniMapOutlineDragging = false;
            }
            else if (_mapDragging)
            {
                Point newScroll = new Point(
                    (int)(2 * (_mapDrag.X - e.X) / _mapScale + _mapDragOffset.X),
                    (int)(2 * (_mapDrag.Y - e.Y) / _mapScale + _mapDragOffset.Y));

                ScrollToPosition(newScroll);
                _mapDragging = false;
            }
            else if (miniMapDragging)
            {
                miniMapRect.X = e.Location.X - miniMapDragOffset.X;
                miniMapRect.Y = e.Location.Y - miniMapDragOffset.Y;

                PlaceMinimapInBounds(false);
                CalculateMinimapAlignment();

                miniMapDragging = false;
                Invalidate();
            }
        }

        protected override bool ProcessDialogKey(Keys keyData)
        {
            switch (keyData)
            {
                case Keys.Up:
                    ScrollToPosition(new Point(sbHorizontal.Value, sbVertical.Value - sbVertical.SmallChange));
                    return true;
                case Keys.Down:
                    ScrollToPosition(new Point(sbHorizontal.Value, sbVertical.Value + sbVertical.SmallChange));
                    return true;
                case Keys.Left:
                    ScrollToPosition(new Point(sbHorizontal.Value - sbHorizontal.SmallChange, sbVertical.Value));
                    return true;
                case Keys.Right:
                    ScrollToPosition(new Point(sbHorizontal.Value + sbHorizontal.SmallChange, sbVertical.Value));
                    return true;
                case Keys.ShiftKey | Keys.Shift:
                    if (!miniMapHandlesVisible)
                    {
                        Point mouseCoords = PointToClient(Cursor.Position);
                        Cursor = GetProperCursor(GetMouseMinimapPosition(new Point(mouseCoords.X - Location.X,
                                                                                   mouseCoords.Y - Location.Y)));
                        Invalidate();
                    }
                    miniMapHandlesVisible = true;
                    return false;
                default:
                    return false;
            }
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (e.KeyCode == Keys.ShiftKey)
            {
                miniMapHandlesVisible = false;
                Cursor = Cursors.Default;
                PaintMinimap(CreateGraphics());
            }

            base.OnKeyUp(e);
        }

        #endregion

        #region Map drawing methods

        public bool FilterSystems { get; set; }
        public Dictionary<int, SolarSystem> UnfilteredSystems;
        public List<Vertex> Route { get; set; }

        public enum SystemFilterType
        {
            JumpRange,
            Route,
            UserFilter,
            None
        }

        private SystemFilterType _myType;

        public SystemFilterType FilterType
        {
            get { return _myType; }
            set { _myType = value; }
        }

        private bool IsFilterActive()
        {
            switch (FilterType)
            {
                case SystemFilterType.Route:
                    if (Route != null)
                        return true;
                    return false;
                case SystemFilterType.JumpRange:
                    return true;
                case SystemFilterType.UserFilter:
                    if (UnfilteredSystems != null)
                        return true;
                    return false;
                default:
                    return false;
            }
        }

        private void DrawSystems(Graphics g, Rectangle clipRect)
        {
            bool filterSystems = IsFilterActive();
            int clipInflateSize = (int)(75 * _mapScale);
            // compensates for regional and constellational edges with labels sticking out
            Rectangle clipInflate = Rectangle.Inflate(clipRect, clipInflateSize, clipInflateSize);

            foreach (var curRegion in StaticStore.Instance.Regions)
            {
                if (UserSettings.Instance.ActiveViewportStyle.ShowRegionNames)
                {
                    PointF drawPos = Utility.ScalePoint(new PointF(
                                                            curRegion.Bounds.Left + curRegion.Bounds.Width / 2,
                                                            curRegion.Bounds.Top + curRegion.Bounds.Height / 2),
                                                        _adjust, _mapScale);
                    DrawTerritoryLabel(g, drawPos, curRegion.Color, _regionFont, curRegion.Name);
                }


                if (Utility.IsAreaVisible(curRegion.Bounds, clipInflate, _adjust, _mapScale, false))
                {
                    foreach (var curConst in curRegion.Constellations)
                    {
                        if (Utility.IsAreaVisible(curConst.Bounds, clipInflate, _adjust, _mapScale, false))
                        {
                            if (UserSettings.Instance.ActiveViewportStyle.ShowConstNames)
                            {
                                PointF drawPos = Utility.ScalePoint(new PointF(
                                                                        curConst.Bounds.Left + curConst.Bounds.Width / 2,
                                                                        curConst.Bounds.Top + curConst.Bounds.Height / 2),
                                                                    _adjust, _mapScale);
                                DrawTerritoryLabel(g, drawPos, curConst.Color, _constFont,
                                                   curConst.Name);
                            }

                            foreach (var curSystem in curConst.Systems)
                                DrawSystemElements(g, curSystem, filterSystems);
                        }
                    }
                }
            }
        }

        private void DrawConnections(Graphics g, Rectangle clipRect)
        {
            DrawGateJumps(clipRect, g);

            if (Route != null)
                DrawRoute(g);

            if (UserSettings.Instance.ActiveViewportStyle.ShowJumpbridges)
                DrawJumpBridges(g, clipRect);

            if (UserSettings.Instance.ActiveViewportStyle.ShowCynoGenerators)
                DrawCynoGens(g);
        }

        private void DrawRoute(Graphics g)
        {
            Pen routePen = new Pen(UserSettings.Instance.ActiveColorScheme.GateRouteColor, _lineThickness);
            Pen routeJumpPen = new Pen(Color.FromArgb(170, UserSettings.Instance.ActiveColorScheme.JumpRouteColor),
                                       _lineThickness);
            Pen routeBridgePen =
                new Pen(Color.FromArgb(170, UserSettings.Instance.ActiveColorScheme.JumpBridgeRouteColor),
                        _lineThickness);

            float scaledNoDraw = (_mapScale < 0.5f)
                                     ? 0
                                     : UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale;
            float drawSizeExternal = (UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize +
                                      UserSettings.Instance.ActiveViewportStyle.SystemMarkerThickness + 6) *
                                     ((_mapScale < 0.60f) ? 0.60f : _mapScale);

            PointF start;
            PointF end;

            for (int i = 0; i < Route.Count - 1; i++)
            {
                start = Utility.ScalePoint(Route[i].SolarSystem.OrganicCoordinates, _adjust,
                                           _mapScale);
                end = Utility.ScalePoint(Route[i + 1].SolarSystem.OrganicCoordinates, _adjust,
                                         _mapScale);
                switch (Route[i + 1].JumpType)
                {
                    case JumpType.Undefined:
                    case JumpType.Gate:
                        g.DrawLine(routePen, start, end);
                        break;
                    case JumpType.Bridge:
                        Bezier.DrawBezier(g, routeBridgePen, start, end, scaledNoDraw, 3, Math.PI / 8);
                        break;

                    case JumpType.Beacon:
                    case JumpType.Cyno:
                        g.DrawEllipse(routeJumpPen, start.X - drawSizeExternal / 2, start.Y - drawSizeExternal / 2,
                                      drawSizeExternal, drawSizeExternal);
                        Bezier.DrawBezier(g, routeJumpPen, start, end, scaledNoDraw, 3, Math.PI / 8);
                        g.DrawEllipse(routeJumpPen, end.X - drawSizeExternal / 2, end.Y - drawSizeExternal / 2,
                                      drawSizeExternal, drawSizeExternal);
                        break;

                    default:
                        throw new Exception("You broke the gate renderer.  I don't know how....");
                }
            }
        }

        private void DrawGateJumps(Rectangle clipRect, Graphics g)
        {
            float scaledNoDraw = (_mapScale < 0.5f)
                                     ? 0
                                     : UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale;
            bool filter = IsFilterActive();

            Color normalGateColor = Color.FromArgb((filter) ? 0x40 : _lineAlpha,
                                                   UserSettings.Instance.ActiveColorScheme.GateGenericColor);
            Color constGateColor = Color.FromArgb((filter) ? 0x40 : _lineAlpha,
                                                  UserSettings.Instance.ActiveColorScheme.GateConstellationColor);
            Color regionGateColor = Color.FromArgb((filter) ? 0x40 : _lineAlpha,
                                                   UserSettings.Instance.ActiveColorScheme.GateRegionColor);

            Pen normalPen = new Pen(normalGateColor, _lineThickness);
            Pen constellationPen = new Pen(constGateColor, _lineThickness);
            Pen regionPen = new Pen(regionGateColor, _lineThickness);
            Pen emptyPen = new Pen(Color.Empty, _lineThickness);

            PointF ScaledFromGate, ScaledToGate;
            Pen gatePen;

            for (int i = 0; i < StaticStore.Instance.StarGates.Count; i++)
            {
                StarGate gate = StaticStore.Instance.StarGates[i];

                ScaledFromGate = Utility.ScalePoint(gate.From.OrganicCoordinates, _adjust, _mapScale);
                ScaledToGate = Utility.ScalePoint(gate.To.OrganicCoordinates, _adjust, _mapScale);

                switch (gate.Type)
                {
                    case GateType.Normal:
                        gatePen = normalPen;
                        break;
                    case GateType.InterConstellation:
                        gatePen = constellationPen;
                        break;
                    case GateType.InterRegion:
                        gatePen = regionPen;
                        break;
                    default:
                        gatePen = emptyPen;
                        break;
                }

                if (Utility.IsLineVisible(ScaledFromGate, ScaledToGate, clipRect, _adjust, _mapScale))
                    DrawGateLine(g, gatePen, gate, scaledNoDraw);
            }
        }

        private void DrawJumpBridges(Graphics g, Rectangle viewRect)
        {
            float scaledNoDraw = (_mapScale < 0.5f)
                                     ? 0
                                     : UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale;
            float circleRadius = UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale;

            foreach (var curJump in DynamicStore.Instance.JumpBridges)
            {
                PointF fromSystem = Utility.ScalePoint(curJump.From.OrganicCoordinates, _adjust, _mapScale);
                PointF toSystem = Utility.ScalePoint(curJump.To.OrganicCoordinates, _adjust, _mapScale);

                if (!Utility.IsLineVisible(fromSystem, toSystem, viewRect, _adjust, _mapScale))
                    continue;

                Pen bridgePen;
                if (IsFilterActive())
                    bridgePen = new Pen(Color.FromArgb(40, curJump.BridgeLine), _lineThickness) { DashStyle = DashStyle.Dash };
                else
                    bridgePen = new Pen(Color.FromArgb(170, curJump.BridgeLine), _lineThickness) { DashStyle = DashStyle.Dash };

                DrawBezier(g, bridgePen, curJump.From.OrganicCoordinates,
                           curJump.To.OrganicCoordinates,
                           scaledNoDraw, 3, Math.PI / 12);

                g.DrawEllipse(bridgePen, fromSystem.X - circleRadius, fromSystem.Y - circleRadius, circleRadius * 2,
                              circleRadius * 2);
                g.DrawEllipse(bridgePen, toSystem.X - circleRadius, toSystem.Y - circleRadius, circleRadius * 2,
                              circleRadius * 2);
            }
        }

        private void DrawCynoGens(Graphics g)
        {
            bool filter = IsFilterActive();

            foreach (var cynoArray in DynamicStore.Instance.CynoBeacons)
            {
                float drawSizeInternal = (UserSettings.Instance.ActiveViewportStyle.SystemMarkerDiameter) *
                                         ((_mapScale < 0.60f) ? 0.60f : _mapScale);
                float drawSizeExternal = (UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize +
                                          UserSettings.Instance.ActiveViewportStyle.SystemMarkerThickness) *
                                         ((_mapScale < 0.60f) ? 0.60f : _mapScale);

                PointF cynoSystem = Utility.ScalePoint(
                    cynoArray.Value.System.OrganicCoordinates, _adjust, _mapScale);

                Color cynoColor = Color.FromArgb((filter) ? 0xFF : 0x40,
                                                 UserSettings.Instance.ActiveColorScheme.CynoBeaconColor);

                using (SolidBrush internalBrush = new SolidBrush(cynoColor))
                    g.FillEllipse(internalBrush, cynoSystem.X - drawSizeInternal / 2, cynoSystem.Y - drawSizeInternal / 2,
                                  drawSizeInternal, drawSizeInternal);

                using (
                    Pen externalPen = new Pen(cynoColor,
                                              UserSettings.Instance.ActiveViewportStyle.SystemMarkerThickness * _mapScale -
                                              1))
                    g.DrawEllipse(externalPen, cynoSystem.X - drawSizeExternal / 2, cynoSystem.Y - drawSizeExternal / 2,
                                  drawSizeExternal, drawSizeExternal);
            }
        }

        public FilterList UserFilter { get; set; }

        private bool SystemIsUnfiltered(SolarSystem system)
        {
            if (UnfilteredSystems != null && UnfilteredSystems.ContainsKey(system.ID))
                return true;

            if (Route == null)
                return false;

            foreach (var route in Route)
            {
                if (route.SolarSystem == system)
                    return true;
            }

            return false;
        }

        private void DrawSystemElements(Graphics g, SolarSystem system, bool filtering)
        {
            PointF sysLoc = Utility.ScalePoint(system.OrganicCoordinates, _adjust, _mapScale);
            bool filterpass;
            if (filtering)
                filterpass = SystemIsUnfiltered(system);
            else filterpass = true;

            DrawSystemMarker(g, system, sysLoc, filterpass);

            if (UserSettings.Instance.ActiveViewportStyle.ShowSovCircles)
                DrawInfoCircle(g, system, sysLoc);
            if (UserSettings.Instance.ActiveViewportStyle.ShowSystemNames &&
                (_mapScale >= 0.50f ||
                 (UserSettings.Instance.ActiveViewportStyle.ShowCynoGenerators &&
                  DynamicStore.Instance.CynoBeacons.ContainsKey(system))))
                DrawSystemLabel(g, system, sysLoc, filterpass);
        }

        private void DrawSystemMarker(Graphics g, SolarSystem system, PointF drawPos, bool passesFilter)
        {
            using (
                Pen systemPen =
                    new Pen(
                        Color.FromArgb((passesFilter) ? 0xFF : 0x40,
                                       SolarSystemColorSchemes.SecurityScheme.GetSystemColor(system)),
                        UserSettings.Instance.ActiveViewportStyle.SystemMarkerThickness * _mapScale))
            {
                float drawSize; // right now all system markers have to have square proportions.
                if (system == _selectedSystem)
                {
                    drawSize = (UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize - 2) * _mapScale;

                    if ((system.Stations != null && system.Stations.Count > 0) ||
                        DynamicStore.Instance.ApiData.ConquerableStations.ContainsKey(system.ID))
                    {
                        g.DrawRectangle(systemPen, drawPos.X - drawSize / 2, drawPos.Y - drawSize / 2, drawSize,
                                        drawSize);
                    }
                    else
                    {
                        g.DrawEllipse(systemPen, drawPos.X - drawSize / 2, drawPos.Y - drawSize / 2, drawSize,
                                      drawSize);
                    }
                }
                else
                {
                    drawSize = UserSettings.Instance.ActiveViewportStyle.SystemMarkerDiameter * _mapScale;

                    if ((system.Stations != null && system.Stations.Count > 0) ||
                        DynamicStore.Instance.ApiData.ConquerableStations.ContainsKey(system.ID))
                    {
                        g.FillRectangle(systemPen.Brush, drawPos.X - drawSize / 2 - _mapScale,
                                        drawPos.Y - drawSize / 2 - _mapScale, drawSize + 2 * _mapScale,
                                        drawSize + 2 * _mapScale);
                    }
                    else
                    {
                        g.DrawEllipse(systemPen, drawPos.X - drawSize / 2, drawPos.Y - drawSize / 2, drawSize,
                                      drawSize);
                    }
                }
            }
            //}
        }

        private void DrawInfoCircle(Graphics g, SolarSystem system, PointF drawPos)
        {
            Sovereignty systemSov = DynamicStore.Instance.ApiData.MapSovereignty[system.ID];
            if (systemSov.AllianceId != 0)
            {
                float radius = UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale * 3;
                RectangleF visibilityRect = new RectangleF(drawPos.X - radius, drawPos.Y - radius, radius * 2, radius * 2);


                using (
                    SolidBrush sovBrush =
                        new SolidBrush(DynamicStore.Instance.ApiData.Alliances[systemSov.AllianceId].Color))
                    g.FillEllipse(sovBrush, visibilityRect);
            }
        }

        private void DrawSystemLabel(Graphics g, SolarSystem system, PointF drawPos, bool passesFilter)
        {
            bool doDetails = UserSettings.Instance.ActiveViewportStyle.ShowSystemDetails && _mapScale > 0.80f;
            Font systemTextFont = system == _selectedSystem ? _systemBoldFont : _systemFont;

            string systemTitle = system.Name;
            string systemDetails;


            if (doDetails)
                systemDetails = GetDetailString(system, UserSettings.Instance.ActiveViewportStyle.DetailText);
            else
                systemDetails = string.Empty;

            SizeF titleSize = g.MeasureString(systemTitle, systemTextFont);
            SizeF detailSize = g.MeasureString(systemDetails, _systemDetailFont);

            SizeF totalSize = new SizeF(titleSize.Width, titleSize.Height + detailSize.Height + 2 * _mapScale);

            bool detailLarger = false;
            if (detailSize.Width > titleSize.Width)
            {
                totalSize.Width = detailSize.Width;
                detailLarger = true;
            }

            float offset = UserSettings.Instance.ActiveViewportStyle.SystemMarkerSize * _mapScale / 2;

            if (_mapScale < 0.50f && UserSettings.Instance.ActiveViewportStyle.ShowCynoGenerators &&
                DynamicStore.Instance.CynoBeacons.ContainsKey(system))
                offset = 3;

            switch (system.FlatLabel)
            {
                case RelativePosition.Top:
                    drawPos.X -= totalSize.Width / 2;
                    drawPos.Y -= totalSize.Height;
                    break;
                case RelativePosition.TopLeft:
                    drawPos.X -= totalSize.Width + offset - 2;
                    drawPos.Y -= totalSize.Height;
                    break;
                case RelativePosition.TopRight:
                    drawPos.X += offset + 1;
                    drawPos.Y -= totalSize.Height;
                    break;
                case RelativePosition.CenterLeft:
                    drawPos.X -= totalSize.Width + offset - 2;
                    drawPos.Y -= totalSize.Height / 2;
                    break;
                case RelativePosition.CenterRight:
                    drawPos.X += offset + 1;
                    drawPos.Y -= totalSize.Height / 2;
                    break;
                case RelativePosition.Bottom:
                    drawPos.X -= totalSize.Width / 2;
                    drawPos.Y += offset;
                    break;
                case RelativePosition.BottomLeft:
                    drawPos.X -= totalSize.Width + offset - 2;
                    drawPos.Y += offset;
                    break;
                case RelativePosition.BottomRight:
                    drawPos.X += offset + 1;
                    drawPos.Y += offset;
                    break;
            }

            PointF titlePos = drawPos, detailPos = drawPos;
            if (detailLarger)
            {
                titlePos.X = drawPos.X + (detailSize.Width - titleSize.Width) / 2;
                detailPos.Y = drawPos.Y + titleSize.Height - 2 * _mapScale;
            }
            else if (UserSettings.Instance.ActiveViewportStyle.ShowSystemDetails)
            {
                detailPos.X = drawPos.X + (titleSize.Width - detailSize.Width) / 2;
                detailPos.Y = drawPos.Y + titleSize.Height - 2 * _mapScale;
            }

            using (
                SolidBrush systemFontBrush =
                    new SolidBrush(Color.FromArgb((passesFilter) ? 0xFF : 0x40,
                                                  UserSettings.Instance.ActiveColorScheme.SystemTextColor)))
                g.DrawString(systemTitle, systemTextFont, systemFontBrush, titlePos);

            if (doDetails)
            {
                Color secColor;

                if (system.Security < 0) secColor = UserSettings.Instance.ActiveColorScheme.NullSecDetailColor;
                else if (system.Security < 0.45) secColor = UserSettings.Instance.ActiveColorScheme.LowSecDetailColor;
                else secColor = UserSettings.Instance.ActiveColorScheme.HighSecDetailColor;

                if (!passesFilter) secColor = Color.FromArgb(0x40, secColor);

                using (SolidBrush secBrush = new SolidBrush(secColor))
                    g.DrawString(systemDetails, _systemDetailFont, secBrush, detailPos);
            }
        }

        private static string GetDetailString(SolarSystem srcSystem, EVEMapDetailText detailParameters)
        {
            string systemDetails = srcSystem.Security.ToString("F", _ni);

            int count = 0;
            switch (detailParameters)
            {
                case EVEMapDetailText.StarbaseStatus:
                    foreach (var starbase in DynamicStore.Instance.ApiData.Starbases)
                    {
                        int poscount = 0;
                        TimeSpan minTime = TimeSpan.Zero;
                        TimeSpan maxTime = TimeSpan.Zero;

                        if (starbase.Value.SolarSystemID == srcSystem.ID)
                        {
                            poscount++;
                            if (minTime == TimeSpan.Zero || starbase.Value.RunTime < minTime)
                                minTime = starbase.Value.RunTime;
                            if (maxTime == TimeSpan.Zero || starbase.Value.RunTime > maxTime)
                                maxTime = starbase.Value.RunTime;
                        }
                        systemDetails = string.Format("[Pos Time: {0}:{1}]", minTime.Hours, minTime.Minutes);
                    }
                    break;
                case EVEMapDetailText.AsteroidBelts:
                    foreach (var planet in srcSystem.Planets)
                        count += planet.Belts;
                    if (count > 0) systemDetails += " (" + count + ")";
                    break;
                case EVEMapDetailText.IceFields:
                    foreach (var planet in srcSystem.Planets)
                        count += planet.IceFields;
                    if (count > 0) systemDetails += " (" + count + ")";
                    break;

                case EVEMapDetailText.OreClass:
                    systemDetails += " (" + srcSystem.SecurityClass + ")";
                    break;
                case EVEMapDetailText.Stations:
                    if (srcSystem.Stations != null && srcSystem.Stations.Count > 0)
                        systemDetails += " (" + srcSystem.Stations.Count + ")";
                    else
                        systemDetails += " (0)";
                    break;
                case EVEMapDetailText.Moons:
                    foreach (var planet in srcSystem.Planets)
                        count += planet.Moons;
                    if (count > 0) systemDetails += " (" + count + ")";
                    break;
                case EVEMapDetailText.Sovereignty:
                    if (DynamicStore.Instance.ApiData.MapSovereignty.ContainsKey(srcSystem.ID))
                    {
                        if (DynamicStore.Instance.ApiData.MapSovereignty[srcSystem.ID].AllianceId == 0)
                            break;
                        systemDetails = "[" +
                                        DynamicStore.Instance.ApiData.Alliances[
                                            DynamicStore.Instance.ApiData.MapSovereignty[srcSystem.ID].AllianceId].
                                            Ticker + "]";
                    }
                    break;
                case EVEMapDetailText.Kills:
                    if (DynamicStore.Instance.ApiData.MapKills.ContainsKey(srcSystem.ID))
                    {
                        Kills info = DynamicStore.Instance.ApiData.MapKills[srcSystem.ID];
                        int totalKills = info.PodKills + info.ShipKills +
                                         info.FactionKills;
                        systemDetails = totalKills + " (P:" + info.PodKills + "  S:" +
                                        info.ShipKills + "  N:" + info.FactionKills + ")";
                    }
                    break;
                case EVEMapDetailText.Jumps:
                    if (DynamicStore.Instance.ApiData.MapJumps.ContainsKey(srcSystem.ID))
                        systemDetails = "( " + DynamicStore.Instance.ApiData.MapJumps[srcSystem.ID].ShipJumps + " )";
                    break;
                case EVEMapDetailText.Rats:

                    foreach (var planet in srcSystem.Planets)
                        count += planet.Belts;
                    systemDetails += " (" + count + ")";

                    EveRatType currentRatType = EveRats.GetRatType(srcSystem.RegionID);
                    if (currentRatType != EveRatType.None)
                        systemDetails += " " + EveRats.RatAbbreviations[currentRatType];
                    break;
            }

            return systemDetails;
        }

        private void DrawTerritoryLabel(Graphics g, PointF position, Color color, Font font, string text)
        {
            SizeF nameSize = g.MeasureString(text, font);
            PointF drawOrigin = new PointF(position.X - nameSize.Width / 2, position.Y - nameSize.Height / 2);

            SolidBrush labelBrush = new SolidBrush(Color.FromArgb(_labelAlpha, color));
            g.DrawString(text, font, labelBrush, drawOrigin);
            labelBrush.Dispose();
        }

        private void DrawGateLine(Graphics g, Pen pen, StarGate srcGate, float nodrawRadius)
        {
            PointF drawPos1 = Utility.ScalePoint(srcGate.From.OrganicCoordinates, _adjust,
                                                 _mapScale);
            PointF drawPos2 = Utility.ScalePoint(srcGate.To.OrganicCoordinates, _adjust, _mapScale);

            if (nodrawRadius > 0) // shorten the line on both ends
            {
                float dx = drawPos2.X - drawPos1.X;
                float dy = drawPos2.Y - drawPos1.Y;
                float length = (float)(Math.Sqrt(dx * dx + dy * dy));

                float xDisplacement = dx / length;
                float yDisplacement = (float)Math.Sqrt(1 - xDisplacement * xDisplacement);

                int xOffset = (int)(nodrawRadius * xDisplacement);
                int yOffset = (int)(nodrawRadius * yDisplacement);

                drawPos1.X += xOffset;
                drawPos2.X -= xOffset;

                drawPos1.Y += (dy > 0) ? yOffset : -yOffset;
                drawPos2.Y -= (dy > 0) ? yOffset : -yOffset;
            }
            g.DrawLine(pen, drawPos1.X, drawPos1.Y, drawPos2.X, drawPos2.Y);
        }

        private void DrawBezier(Graphics g, Pen pen, PointF start, PointF end, float nodrawRadius,
                                float lengthfraction, double angle)
        {
            float drawPosX1 = start.X, drawPosY1 = -start.Y, drawPosX2 = end.X, drawPosY2 = -end.Y;

            float dx = drawPosX2 - drawPosX1;
            float dy = drawPosY2 - drawPosY1;
            if (dx < 0)
            {
                drawPosX1 = end.X;
                drawPosY1 = -end.Y;
                drawPosX2 = start.X;
                drawPosY2 = -start.Y;
                dx = drawPosX2 - drawPosX1;
                dy = drawPosY2 - drawPosY1;
            }
            float length = (float)(Math.Sqrt(dx * dx + dy * dy));

            double alpha = (float)(Math.Acos(dx / length));
            if (dy < 0) // adjust for the right quadrant
            {
                if (dx > 0) alpha = -alpha;
                else alpha = Math.PI + Math.PI - alpha;
            }

            float bx1 = drawPosX1 + length / lengthfraction * (float)Math.Cos(alpha + angle);
            float by1 = drawPosY1 + length / lengthfraction * (float)Math.Sin(alpha + angle);
            float bx2 = drawPosX2 - length / lengthfraction * (float)Math.Cos(alpha - angle);
            float by2 = drawPosY2 - length / lengthfraction * (float)Math.Sin(alpha - angle);

            drawPosX1 += nodrawRadius * (float)Math.Cos(alpha);
            drawPosY1 += nodrawRadius * (float)Math.Sin(alpha);
            drawPosX2 -= nodrawRadius * (float)Math.Cos(alpha);
            drawPosY2 -= nodrawRadius * (float)Math.Sin(alpha);

            PointF src = Utility.ScalePoint(new PointF(drawPosX1, -drawPosY1), _adjust, _mapScale);
            PointF bezier1 = Utility.ScalePoint(new PointF(bx1, -by1), _adjust, _mapScale);
            PointF bezier2 = Utility.ScalePoint(new PointF(bx2, -by2), _adjust, _mapScale);
            PointF dest = Utility.ScalePoint(new PointF(drawPosX2, -drawPosY2), _adjust, _mapScale);

            g.DrawBezier(pen, src, bezier1, bezier2, dest);
        }

        #endregion

        #region Minimap drawing methods

        private void MiniMapFillBitmap()
        {
            if (_flatDataChanged)
            {
                GetGlobalFlatArea();
                _flatDataChanged = false;
            }
            //TODO: FIX THIS
            _miniMapBitmap = new Bitmap(miniMapRect.Width, miniMapRect.Height, PixelFormat.Format32bppPArgb);

            Graphics g = Graphics.FromImage(_miniMapBitmap);
            g.SmoothingMode = SmoothingMode.HighQuality;

            g.Clear(UserSettings.Instance.ActiveColorScheme.MinimapBackgroundColor);
            using (Pen borderPen = new Pen(UserSettings.Instance.ActiveColorScheme.MinimapBorderColor, 1))
                g.DrawRectangle(borderPen, 0, 0, miniMapRect.Width - 1, miniMapRect.Height - 1);

            ThreadPool.QueueUserWorkItem(MiniMapGenerateBitmap, _miniMapBitmap.Clone());
            MiniMapDrawWaitIndicator(g);
        }

        private void MiniMapDrawResizeDisplay(Graphics g)
        {
            using (SolidBrush backBrush = new SolidBrush(UserSettings.Instance.ActiveColorScheme.MinimapBackgroundColor)
                )
                g.FillRectangle(backBrush, miniMapRect.Left, miniMapRect.Top, miniMapRect.Width - 1,
                                miniMapRect.Height - 1);
            using (Pen borderPen = new Pen(UserSettings.Instance.ActiveColorScheme.MinimapBorderColor, 1))
                g.DrawRectangle(borderPen, miniMapRect.Left, miniMapRect.Top, miniMapRect.Width - 1,
                                miniMapRect.Height - 1);

            using (Font workingFont = new Font("Verdana", 7, FontStyle.Italic))
            {
                string s = miniMapRect.Width + "x" + miniMapRect.Height;
                SizeF len = g.MeasureString(s, workingFont);

                if (len.Width >= miniMapRect.Width || len.Height >= miniMapRect.Height) return;

                PointF loc = new PointF(miniMapRect.Left + (miniMapRect.Width - len.Width) / 2,
                                        miniMapRect.Top + (miniMapRect.Height - len.Height) / 2);

                using (SolidBrush textBrush = new SolidBrush(Color.Gray))
                    g.DrawString(s, workingFont, textBrush, loc);
            }
        }

        private void MiniMapGenerateBitmap(object baseBitmap)
        {
            SizeF mapSize = new SizeF(
                _globalFlatArea.Width + 2 * UserSettings.Instance.ActiveViewportStyle.Margin.Width,
                _globalFlatArea.Height + 2 * UserSettings.Instance.ActiveViewportStyle.Margin.Height);

            miniMapScale = new SizeF((miniMapRect.Width - 2) / mapSize.Width, (miniMapRect.Height - 2) / mapSize.Height);
            float miniMapAvgScale = (miniMapScale.Width + miniMapScale.Height) / 2;
            float alphaMultiplier = miniMapAvgScale * 65;

            int systemAlpha = (int)(0x30 * alphaMultiplier), gateAlpha = (int)(0x60 * alphaMultiplier);
            if (systemAlpha > 0xFF) systemAlpha = 0xFF;
            if (gateAlpha > 0xFF) gateAlpha = 0xFF;

            Graphics g = Graphics.FromImage((Bitmap)baseBitmap);
            g.SmoothingMode = SmoothingMode.HighQuality;

            Pen jumpPenNormal =
                new Pen(Color.FromArgb(Math.Abs(gateAlpha), UserSettings.Instance.ActiveColorScheme.GateGenericColor), 1);
            Pen jumpPenConst = new Pen(UserSettings.Instance.ActiveColorScheme.GateConstellationColor, 1);
            Pen jumpPenRegion = new Pen(UserSettings.Instance.ActiveColorScheme.GateRegionColor, 1);

            foreach (var gate in StaticStore.Instance.StarGates)
            {
                SolarSystem fromSystem = gate.From;
                SolarSystem toSystem = gate.To;
                float dx = fromSystem.OrganicCoordinates.X - toSystem.OrganicCoordinates.X;
                float dy = fromSystem.OrganicCoordinates.Y - toSystem.OrganicCoordinates.Y;
                float length = (float)Math.Sqrt(dx * dx + dy * dy);

                if (length * miniMapAvgScale > 1)
                {
                    Pen gatepen = null;
                    switch (gate.Type)
                    {
                        case GateType.Normal:
                            gatepen = jumpPenNormal;
                            break;
                        case GateType.InterConstellation:
                            gatepen = jumpPenConst;
                            break;
                        case GateType.InterRegion:
                            gatepen = jumpPenRegion;
                            break;
                    }

                    float drawPosX1 = (fromSystem.OrganicCoordinates.X - _globalFlatArea.Left +
                                       UserSettings.Instance.ActiveViewportStyle.Margin.Width) * miniMapScale.Width,
                          drawPosY1 = (fromSystem.OrganicCoordinates.Y - _globalFlatArea.Top +
                                       UserSettings.Instance.ActiveViewportStyle.Margin.Height) * miniMapScale.Height,
                          drawPosX2 = (toSystem.OrganicCoordinates.X - _globalFlatArea.Left +
                                       UserSettings.Instance.ActiveViewportStyle.Margin.Width) * miniMapScale.Width,
                          drawPosY2 = (toSystem.OrganicCoordinates.Y - _globalFlatArea.Top +
                                       UserSettings.Instance.ActiveViewportStyle.Margin.Height) * miniMapScale.Height;


                    if (gatepen != null) g.DrawLine(gatepen, drawPosX1, drawPosY1, drawPosX2, drawPosY2);
                }
            }

            foreach (var curRegion in StaticStore.Instance.Regions)
            {
                using (Pen systemPen = new Pen(Color.FromArgb(Math.Abs(systemAlpha), curRegion.Color)))
                {
                    foreach (var curConst in curRegion.Constellations)
                        foreach (var curSystem in curConst.Systems)
                        {
                            int drawPosX =
                                (int)
                                ((curSystem.OrganicCoordinates.X - _globalFlatArea.Left +
                                  UserSettings.Instance.ActiveViewportStyle.Margin.Width) *
                                 miniMapScale.Width),
                                drawPosY =
                                    (int)
                                    ((curSystem.OrganicCoordinates.Y - _globalFlatArea.Top +
                                      UserSettings.Instance.ActiveViewportStyle.Margin.Height) *
                                     miniMapScale.Height);

                            g.DrawEllipse(systemPen, drawPosX, drawPosY, 1, 1);
                            // this is a lot better looking, and speed isn't a big deal
                            //_miniMapBitmap.SetPixel((int)drawPosX, (int)drawPosY, Color.FromArgb(0xA0, curRegion.Color));
                        }
                }
            }

            jumpPenNormal.Dispose();
            jumpPenConst.Dispose();
            jumpPenRegion.Dispose();
            g.Dispose();

            _miniMapBitmap = ((Bitmap)baseBitmap).Clone() as Bitmap;
            Invalidate();
        }

        private void MiniMapDrawWaitIndicator(Graphics g)
        {
            string[] text = new[] { "Generating minimap...", "Working...", "..." };

            using (Font workingFont = new Font("Verdana", 7, FontStyle.Italic))
            {
                for (int i = 0; i < text.Length; i++)
                {
                    string s = text[i];
                    SizeF len = g.MeasureString(s, workingFont);
                    if (len.Width >= miniMapRect.Width || len.Height >= miniMapRect.Height) continue;

                    PointF loc = new PointF((miniMapRect.Width - len.Width) / 2,
                                            (miniMapRect.Height - len.Height) / 2);

                    using (SolidBrush textBrush = new SolidBrush(Color.Gray))
                        g.DrawString(s, workingFont, textBrush, loc);

                    return;
                }
            }
        }

        private void MiniMapDrawOutline(Graphics g)
        {
            Point origin = new Point(
                (int)(sbHorizontal.Value * miniMapScale.Width + miniMapRect.Left + 1),
                (int)(sbVertical.Value * miniMapScale.Height + miniMapRect.Top + 1));
            Size size = new Size(
                (int)(_mapEffectiveSize.Width / (sbHorizontal.Maximum * _mapScale) * (miniMapRect.Width - 2)),
                (int)(_mapEffectiveSize.Height / (sbVertical.Maximum * _mapScale) * (miniMapRect.Height - 2)));

            if (size.Width > miniMapRect.Width - 3) size.Width = miniMapRect.Width - 3;
            if (size.Height > miniMapRect.Height - 3) size.Height = miniMapRect.Height - 3;

            miniMapOutlineRect = new Rectangle(origin, size);

            using (
                SolidBrush outlineBrush =
                    new SolidBrush(UserSettings.Instance.ActiveColorScheme.MinimapFocusBackgroundColor))
                g.FillRectangle(outlineBrush, miniMapOutlineRect);

            using (Pen outlinePen = new Pen(UserSettings.Instance.ActiveColorScheme.MinimapFocusOutlineColor))
                g.DrawRectangle(outlinePen, miniMapOutlineRect);
        }

        #endregion

    }
}