using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using libEveMapNG.Data.Static;

namespace EveMapControls
{
    public partial class SystemList : UserControl
    {
        private static readonly char[] dndSeparators = new[] { ',', '\n', '\t' };
        private bool allowDuplicates;
        private string groupName;

        public SystemList()
        {
            InitializeComponent();
            ssbSystem.LimitToJumpDestinations = LimitToJumpDestinations;
        }

        public string GroupName
        {
            get { return groupName; }
            set
            {
                groupName = value;
                gbTitle.Text = value;
            }
        }

        public bool LimitToJumpDestinations { get; set; }
        
        public List<SolarSystem> SelectedSystems
        {
            get
            {
                List<SolarSystem> mySystems = new List<SolarSystem>();
                foreach (String system in lbSystems.Items)
                {
                    mySystems.Add(SolarSystem.GetByName(system));
                }
                return mySystems;
            }
        }

        public string GroupBoxName
        {
            set { gbTitle.Text = value; }
        }

        public bool AllowDuplicates
        {
            set { allowDuplicates = value; }
            get { return allowDuplicates; }
        }



        // PUBLIC FUNCTIONS //

        public void Clear()
        {
            lbSystems.Items.Clear();
        }

        public bool Contains(String item)
        {
            foreach (var existing in lbSystems.Items)
                if (item.Equals(existing))
                    return true;
            return false;
        }


        // EVENTS //

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (ssbSystem.Text != "" && (SolarSystem)ssbSystem.SelectedItem != null)
                lbSystems.Items.Add(((SolarSystem)ssbSystem.SelectedItem).Name);
            // else
            //     lbSystems.Items.Add(ssbSystem.Text);
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            Clear();
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            while (lbSystems.SelectedIndices.Count > 0)
            {
                lbSystems.Items.RemoveAt(lbSystems.SelectedIndices[0]);
            }
        }

        // kick off drag and drop
        private void lbSystems_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.None) return;

            int location = lbSystems.IndexFromPoint(e.X, e.Y);
            if (location < 0 || location > lbSystems.Items.Count) return;

            ListBox.SelectedObjectCollection objects = lbSystems.SelectedItems;
            if (objects.Count == 0) return;

            ListBox.SelectedIndexCollection indicies = lbSystems.SelectedIndices;
            DragAndDropData data = new DragAndDropData(indicies, objects);
            lbSystems.DoDragDrop(data, DragDropEffects.Move | DragDropEffects.Copy);
        }

        // recieve drag and drop data
        private void lbSystems_DragDrop(object sender, DragEventArgs e)
        {
            // marshall
            IList inputs = new List<String>();
            Type format = typeof(DragAndDropData);
            DragAndDropData data = null;
            if (e.Data.GetDataPresent(format))
            {
                data = (DragAndDropData)e.Data.GetData(format);
                foreach (var item in data.objects)
                    inputs.Add(item);
            }
            else if (e.Data.GetDataPresent(typeof(String)))
                AddRangeToList(((String)e.Data.GetData(typeof(String))).Split(dndSeparators), inputs);
            else if (e.Data.GetDataPresent(typeof(String[])))
                AddRangeToList((String[])e.Data.GetData(typeof(String[])), inputs);
            if (inputs.Count == 0) return;

            // validate
            foreach (var item in inputs)
            {
                if (SolarSystem.GetByName((String)item) == null)
                    inputs.Remove(item);
                if (sender != lbSystems && !allowDuplicates && Contains((String)item))
                    inputs.Remove(item);
            }
            if (inputs.Count == 0) return;

            // location to insert into
            Point localPoint = lbSystems.PointToClient(new Point(e.X, e.Y));
            int location = lbSystems.IndexFromPoint(localPoint);
            if (location < 0 || location > lbSystems.Items.Count) location = lbSystems.Items.Count;

            // remove self same points for MOVE and adjust location
            const int trapped = 0;
            if (data != null && sender == lbSystems)
            {
                while (data.indicies.Count > 0)
                {
                    int index = data.indicies[0];
                    lbSystems.Items.RemoveAt(index);
                    if (location > index) location--;
                }
            }
            location -= trapped;

            // insert
            foreach (var input in inputs)
            {
                lbSystems.Items.Insert(location, input);
                location++;
            }
        }

        private void lbSystems_DragOver(object sender, DragEventArgs e)
        {
            e.Effect = (sender == lbSystems) ? DragDropEffects.Move : DragDropEffects.Copy;
        }

        private void lbSystems_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Up:
                case Keys.Down:
                    if (!e.Control) break;
                    bool up = e.KeyCode == Keys.Up;

                    // move elements
                    List<int> selected = new List<int>();
                    foreach (int i in lbSystems.SelectedIndices)
                    {
                        selected.Add(i);

                        String item = (String)lbSystems.Items[i];
                        lbSystems.Items.RemoveAt(i);
                        int location = RangeCheck(((up) ? -1 : 1) + i);
                        lbSystems.Items.Insert(location, item);
                    }

                    // move selection
                    lbSystems.SelectedIndices.Clear();
                    foreach (int i in selected)
                        lbSystems.SelectedIndices.Add(RangeCheck(i));
                    break;

                case Keys.Delete:
                    foreach (int item in lbSystems.SelectedIndices)
                        lbSystems.Items.RemoveAt(item);
                    break;
            }
        }


        // PRIVATE HELPERS //

        private static void AddRangeToList(IEnumerable items, IList list)
        {
            foreach (var item in items)
                list.Add(item);
        }

        private int RangeCheck(int location)
        {
            if (location > lbSystems.Items.Count) location = lbSystems.Items.Count;
            else if (location < 0) location = 0;
            return location;
        }

        public void Add(SolarSystem solarSystem)
        {
            if (!lbSystems.Items.Contains(solarSystem.Name))
                lbSystems.Items.Add(solarSystem.Name);
        }

        #region Nested type: DragAndDropData

        private class DragAndDropData
        {
            public readonly ListBox.SelectedIndexCollection indicies;
            public readonly ListBox.SelectedObjectCollection objects;

            public DragAndDropData(ListBox.SelectedIndexCollection indicies, ListBox.SelectedObjectCollection objects)
            {
                this.objects = objects;
                this.indicies = indicies;
            }
        }

        #endregion
    }
}