using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Types;
using libEveMapNG.Utility;

namespace libEveMapNG.Routes
{
    public class CynoRoute
    {
        #region Delegates

        public delegate void SystemProcessedHandler(object sender, ProgressChangedEventArgs e);

        #endregion

        private const float MAXIMUM_JUMP_DISTANCE = 14.625f;

        private static readonly Dictionary<SolarSystem, Vertex> SystemsList =
            new Dictionary<SolarSystem, Vertex>(StaticStore.Systems.Count);

        private static JumpRoute RouteParameters;
        public static Dictionary<SolarSystem, List<SolarSystem>> StaticAdjacencies; //This list gets serialized 

        private readonly Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node> NodeList =
            new Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node>();

        private readonly FibonacciHeap<Vertex> SystemPriorityQueue = new FibonacciHeap<Vertex>();
        private float PilotMaxRange;

        public CynoRoute()
        {
            //SystemsList is stored in the DynamicStore. If it needs to be regenerated DynamicStore will do that.
            StaticAdjacencies = DynamicStore.Instance.UniverseGraph;

            foreach (var currentSystem in StaticStore.Systems.Values)
            {
                Vertex newVertex = new Vertex
                                       {
                                           SolarSystem = currentSystem,
                                           Cost = float.MaxValue,
                                           LYTraveled = float.MaxValue,
                                       };
                if (!SystemsList.ContainsKey(newVertex.SolarSystem))
                    //Workaround for duplicate systems in the store :lhydaarghfist:
                    SystemsList.Add(newVertex.SolarSystem, newVertex);
            }
        }

        public event SystemProcessedHandler SystemProcessed;

        public List<Vertex> GetRoute(JumpRoute JumpParameters)
        {
            RouteParameters = JumpParameters;
            RouteParameters.WayPoints.Insert(0, JumpParameters.StartSystem); //Add the start to the front
            RouteParameters.WayPoints.Add(JumpParameters.DestSystem); //Add the end to the back
            PilotMaxRange = JumpDrive.GetRange(RouteParameters.ShipProfile.JDC, RouteParameters.ShipProfile.Ship);

            //for each waypoint of the trip, find the shortest route between the next two waypoints and it to the overall route
            var completeRoute = new List<Vertex>();
            for (int i = 0; i < RouteParameters.WayPoints.Count - 1; i++)
            {
                resetGraph();
                completeRoute.AddRange(getShortestPath(RouteParameters.WayPoints[i], RouteParameters.WayPoints[i + 1]));
            }

            //The way we backtrack to incorporate hitting waypoints, we need to insert the start system to be complete.
            completeRoute.Insert(0, SystemsList[RouteParameters.StartSystem]);

            return completeRoute;
        }

        /// <summary>
        /// Finds the shortest path for the route 'Start' to 'End'
        /// using Dijkstras weighted graph shortest path algorithm
        /// </summary>
        /// <param name="Start"></param>
        /// <param name="End"></param>
        /// <returns></returns>
        private List<Vertex> getShortestPath(SolarSystem Start, SolarSystem End)
        {
            Vertex Current;
            SystemsList[Start].Cost = 0;
            SystemsList[Start].LYTraveled = 0;

            SystemPriorityQueue.insert(SystemsList[Start], SystemsList[Start].Cost);

            while (!SystemPriorityQueue.isEmpty())
            {
                Current = SystemPriorityQueue.removeMin().datum;

                /* If you want to do jumpbridge routing options here would be the place to do it */


                foreach (SolarSystem AdjacentID in StaticAdjacencies[Current.SolarSystem])
                {
                    Vertex AdjacentSystem = SystemsList[AdjacentID];

                    float oldcost = SystemsList[AdjacentID].Cost;
                    float RangeToNextSystem = JumpDrive.GetDistance(Current.SolarSystem, AdjacentID);
                    if (RangeToNextSystem > PilotMaxRange)
                        continue;

                    CostWrapper NextJump = GetCost(Current.SolarSystem, AdjacentID, RangeToNextSystem);
                    if (Current.Cost + NextJump.Cost < AdjacentSystem.Cost ||
                        (Current.Cost + NextJump.Cost == AdjacentSystem.Cost &&
                         Current.LYTraveled + RangeToNextSystem < AdjacentSystem.LYTraveled))
                    {
                        /* Update the algorithm variables */
                        AdjacentSystem.Cost = Current.Cost + NextJump.Cost;
                        AdjacentSystem.LYTraveled = Current.LYTraveled + RangeToNextSystem;
                        AdjacentSystem.Path = Current.SolarSystem;
                        AdjacentSystem.FuelCost = NextJump.FuelCost;

                        AdjacentSystem.JumpType = NextJump.JumpType;
                        //Current.FuelCost = NextJump.FuelCost;
                        switch (NextJump.JumpType)
                        {
                            case JumpType.Beacon:
                                AdjacentSystem.Beacon = NextJump.Beacon; //Redundant but whatever.
                                break;
                            case JumpType.Bridge:
                                //AdjacentSystem.Bridge = NextJump.Bridge; //TBD.
                                break;
                            default:
                                break;
                        }


                        if (oldcost == float.MaxValue)
                            NodeList[AdjacentID] = SystemPriorityQueue.insert(AdjacentSystem, AdjacentSystem.Cost);
                        else
                            SystemPriorityQueue.decreaseKey(NodeList[AdjacentID], AdjacentSystem.Cost);
                    }
                }

                //Processed a system, raise an event.
                SystemProcessed(null, null);
            }
            //Once Dijkstras finds the path, start at the end and trace backwards

            var SystemsRoute = new List<Vertex>();
            Vertex BackTrack = SystemsList[End].Copy();

            while (BackTrack.SolarSystem != Start)
            {
                SystemsRoute.Insert(0, BackTrack);
                BackTrack = SystemsList[BackTrack.Path].Copy();
            }

            return SystemsRoute;
        }

        private void resetGraph()
        {
            foreach (var v in SystemsList.Values)
            {
                v.Cost = float.MaxValue;
                v.LYTraveled = float.MaxValue;
            }
            NodeList.Clear();
            SystemPriorityQueue.clear();
        }

        //Based on the route RouteParameters, get the cost it would take to travel to the system.
        //This assumes that the system is within jumprange as determined by GetDistance
        private static CostWrapper GetCost(SolarSystem fromSystem, SolarSystem toSystem, float JumpDistance)
        {
            /* Start with the basic normal cyno and update it if we can find an edge with lower weight */

            CostWrapper EdgeCost = new CostWrapper
                                       {
                                           Cost = EdgeWeights.JumpDefault,
                                           FuelCost = JumpDrive.GetFuel(JumpDistance, RouteParameters),
                                           JumpType = JumpType.Cyno
                                       };

            if (RouteParameters.AvoidSystems.Count > 0 && RouteParameters.AvoidSystems.ContainsKey(toSystem))
            {
                EdgeCost.Cost = EdgeWeights.JumpHigh;
                EdgeCost.JumpType = JumpType.Undefined;
                return EdgeCost;
            }

            if (RouteParameters.useBeacons && DynamicStore.Instance.CynoBeacons.ContainsKey(toSystem))
            {
                /* The system we have been asked to consider has a beacon */
                EdgeCost.Cost = EdgeWeights.JumpBeacon;
                EdgeCost.JumpType = JumpType.Beacon;
                EdgeCost.Beacon = DynamicStore.Instance.CynoBeacons[toSystem];
                EdgeCost.FuelCost = JumpDrive.GetFuel(JumpDistance, RouteParameters);
            }

            /* Bridges go here */

            /* Cynojammers in the future */

            if (EdgeWeights.JumpStation < EdgeCost.Cost && toSystem.Stations.Count > 0)
            {
                EdgeCost.Cost = EdgeWeights.JumpStation;
                EdgeCost.JumpType = JumpType.Cyno;
                EdgeCost.FuelCost = JumpDrive.GetFuel(JumpDistance, RouteParameters);
            }

            return EdgeCost;
        }

        //Build the List of adjacencies that describes the graph/universe
        public static Dictionary<SolarSystem, List<SolarSystem>> BuildGraph()
        {
            StaticAdjacencies = new Dictionary<SolarSystem, List<SolarSystem>>();

            foreach (SolarSystem currentSystem in StaticStore.Systems.Values)
            {
                StaticAdjacencies.Add(currentSystem, new List<SolarSystem>());
                Constellation CurrentConstellation = Constellation.GetByID(currentSystem.ConstellationID);
                foreach (int AdjConstID in CurrentConstellation.JumpAdjacentConstellations)
                {
                    Constellation nextConst = Constellation.GetByID(AdjConstID);
                    foreach (SolarSystem possibleConnection in nextConst.Systems)
                    {
                        if (possibleConnection.Security > 0.45f)
                            continue;
                        if (JumpDrive.GetDistance(currentSystem, possibleConnection) < MAXIMUM_JUMP_DISTANCE)
                            StaticAdjacencies[currentSystem].Add(possibleConnection);
                    }
                }
            }
            return StaticAdjacencies;
        }
    }
}