using System.Collections.Generic;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Types;
using libEveMapNG.Utility;

namespace libEveMapNG.Routes
{
    public class GateRoute
    {
        // List of every system and all the jump bridges going out of it
        private readonly Dictionary<SolarSystem, List<JumpBridgeEdge>> bridgeList =
            new Dictionary<SolarSystem, List<JumpBridgeEdge>>();

        // Holds references to all the nodes in the priority queue for easy access in decrease key
        private readonly Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node> nodeList =
            new Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node>();

        // Priority queue to dequeue next lowest cost Vertex to consider 
        private readonly FibonacciHeap<Vertex> systemsPriorityQueue = new FibonacciHeap<Vertex>();

        // Master list of ALL the SolarSystems as represented by Vertices
        private readonly Dictionary<SolarSystem, Vertex> vertexList =
            new Dictionary<SolarSystem, Vertex>(StaticStore.Systems.Count);

        private RouteData _parameters;

        /// <summary>
        /// Gets the complete route based off the given Parameters
        /// </summary>
        /// <param name="routeParameters"></param>
        /// <returns></returns>
        public List<Vertex> GetRoute(RouteData routeParameters)
        {
            //Add Start to the beginning, add Destination to the end
            _parameters = routeParameters;
            _parameters.WayPoints.Insert(0, routeParameters.StartSystem);
            _parameters.WayPoints.Add(routeParameters.DestSystem);
            BuildGraph();

            //for each waypoint of the trip, find the shortest route between the next two waypoints and it to the overall route
            var completeRoute = new List<Vertex>();
            for (int i = 0; i < routeParameters.WayPoints.Count - 1; i++)
            {
                ResetGraph();
                completeRoute.AddRange(GetShortestPath(_parameters.WayPoints[i], _parameters.WayPoints[i + 1]));
            }

            //Add the first system just to be complete with final list of systems
            completeRoute.Insert(0, vertexList[routeParameters.StartSystem]);

            return completeRoute;
        }

        /// <summary>
        /// Finds the shortest path for the path 'Start' to 'End'
        /// </summary>
        /// <param name="start">Starting SolarSystem</param>
        /// <param name="end">Ending SolarySystem</param>
        /// <returns>The list of waypoints to take.</returns>
        private List<Vertex> GetShortestPath(SolarSystem start, SolarSystem end)
        {
            Vertex current;
            vertexList[start].Cost = 0;
            systemsPriorityQueue.insert(vertexList[start], vertexList[start].Cost);

            while (!systemsPriorityQueue.isEmpty())
            {
                /* Dequeue the next lowest cost system to consider. */
                current = systemsPriorityQueue.removeMin().datum;

                /* If the current system has bridges out of it, check them */
                if (_parameters.UseBridges &&
                    bridgeList.ContainsKey(current.SolarSystem) &&
                    bridgeList[current.SolarSystem] != null)
                {
                    foreach (JumpBridgeEdge adjacent in bridgeList[current.SolarSystem])
                    {
                        Vertex neighbor = vertexList[adjacent.To];
                        float oldcost = neighbor.Cost;
                        if (current.Cost + EdgeWeights.GateBridge < neighbor.Cost)
                        {
                            neighbor.Cost = current.Cost + EdgeWeights.GateBridge;
                            neighbor.Path = current.SolarSystem;

                            neighbor.FuelCost = JumpDrive.GetJumpBridgeFuel(current.SolarSystem, neighbor.SolarSystem,
                                                                            _parameters.ShipMass);
                            neighbor.JumpType = JumpType.Bridge;
                            neighbor.Bridge = adjacent;

                            if (oldcost == float.MaxValue)
                                nodeList[neighbor.SolarSystem] = systemsPriorityQueue.insert(neighbor, neighbor.Cost);

                            else
                            {
                                systemsPriorityQueue.decreaseKey(nodeList[neighbor.SolarSystem],
                                                                 current.Cost + EdgeWeights.GateBridge);
                            }
                        }
                    }
                }

                /* Check regular gate connections here */
                foreach (Vertex adjacent in current.Adjacencies)
                {
                    float oldcost = adjacent.Cost;
                    CostWrapper nextJump = GetCost(adjacent.SolarSystem);
                    if (current.Cost + nextJump.Cost < adjacent.Cost)
                    {
                        adjacent.Cost = current.Cost + nextJump.Cost;
                        adjacent.Path = current.SolarSystem;
                        adjacent.FuelCost = nextJump.FuelCost;

                        adjacent.JumpType = nextJump.JumpType;

                        if (oldcost == float.MaxValue)
                            nodeList[adjacent.SolarSystem] = systemsPriorityQueue.insert(adjacent, adjacent.Cost);
                        else
                        {
                            systemsPriorityQueue.decreaseKey(nodeList[adjacent.SolarSystem],
                                                             current.Cost + nextJump.Cost);
                        }
                    }
                }
            }
            var completeRoute = new List<Vertex>();

            //Make copies of the vertex since the vertices values will be reset if waypoints are used.
            //Subsequently screwing up all the references the verticies had about the previous path(s).
            Vertex backTrack = vertexList[end].Copy();
            while (backTrack.SolarSystem != start)
            {
                //adding items to the front since were working backwards
                completeRoute.Insert(0, backTrack);
                backTrack = vertexList[backTrack.Path].Copy();
            }

            return completeRoute;
        }

        /// <summary>
        /// Initializes the graph
        /// </summary>
        private void BuildGraph()
        {
            vertexList.Clear();
            nodeList.Clear();
            systemsPriorityQueue.clear();

            foreach (var currentSystem in StaticStore.Systems.Values)
            {
                Vertex newVertex = new Vertex
                                       {
                                           Cost = float.MaxValue,
                                           Adjacencies = new List<Vertex>(),
                                           SolarSystem = SolarSystem.GetByID(currentSystem.ID),
                                           JumpType = JumpType.Gate
                                       };
                vertexList[currentSystem] = newVertex;
            }

            foreach (var gate in StaticStore.Instance.StarGates)
            {
                if (!vertexList[gate.From].Adjacencies.Contains(vertexList[gate.To]))
                {
                    vertexList[gate.From].Adjacencies.Add(vertexList[gate.To]);
                    vertexList[gate.To].Adjacencies.Add(vertexList[gate.From]);
                }
            }

            /* Split the bi-directionaly JumpBridge type into two one-way JumpBridgeEdges for bridge routing */
            foreach (var jumpbridge in DynamicStore.Instance.JumpBridges)
            {
                if (!bridgeList.ContainsKey(jumpbridge.From))
                    bridgeList[jumpbridge.From] = new List<JumpBridgeEdge>();
                if (!bridgeList.ContainsKey(jumpbridge.To))
                    bridgeList[jumpbridge.To] = new List<JumpBridgeEdge>();

                JumpBridgeEdge forwardJump = new JumpBridgeEdge
                                                 {
                                                     From = jumpbridge.From,
                                                     To = jumpbridge.To,
                                                     Pos = jumpbridge.Base,
                                                     Bridge = jumpbridge
                                                 };
                JumpBridgeEdge backwardJump = new JumpBridgeEdge
                                                  {
                                                      From = jumpbridge.To,
                                                      To = jumpbridge.From,
                                                      Pos = jumpbridge.Destination,
                                                      Bridge = jumpbridge
                                                  };

                bridgeList[jumpbridge.From].Add(forwardJump);
                bridgeList[jumpbridge.To].Add(backwardJump);
            }
        }


        /// <summary>
        /// Reset all the costs for each Vertex. Clear the systemsPriorityQueue and its nodes if theres any left over.
        /// </summary>
        private void ResetGraph()
        {
            nodeList.Clear();
            systemsPriorityQueue.clear();
            foreach (var vertex in vertexList.Values)
            {
                //bug After the first vertex, the old path vertices lose their jump types. Fixed?
                //vertex.JumpType = JumpType.Gate;
                vertex.Cost = float.MaxValue;
            }
        }

        /// <summary>
        /// Based on the route _parameters, get the cost it would take to travel to the system.
        /// </summary>
        /// <param name="toSystemId"></param>
        /// <returns></returns>
        private CostWrapper GetCost(SolarSystem toSystemId)
        {
            SolarSystem toSystem = toSystemId;

            CostWrapper edgeCost = new CostWrapper {Cost = EdgeWeights.GateDefault, JumpType = JumpType.Gate};

            /* If we are considering a system thats on the avoid list */
            foreach (var systemToAvoid in _parameters.AvoidSystems)
            {
                if (toSystem.ID == systemToAvoid.ID)
                {
                    edgeCost.Cost = EdgeWeights.GateHigh;
                    return edgeCost;
                }
            }

            /* Checking security status */
            switch (_parameters.RoutingOption)
            {
                case GateRouteOption.AvoidNullSec:
                    if (toSystem.Security <= 0)
                        edgeCost.Cost = EdgeWeights.GateHigh;
                    break;
                case GateRouteOption.AvoidLowSec:
                    if (toSystem.Security < 0.5)
                        edgeCost.Cost = EdgeWeights.GateHigh;
                    break;
                case GateRouteOption.AvoidHighSec:
                    if (toSystem.Security >= 0.5)
                        edgeCost.Cost = EdgeWeights.GateHigh;
                    break;
                case GateRouteOption.None:
                    break;
            }

            return edgeCost;
        }
    }
}