using System.Collections.Generic;
using System.ComponentModel;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Types;
using libEveMapNG.Utility;

namespace libEveMapNG.Routes
{
    public class JumpRouter
    {
        #region Delegates

        public delegate void SystemProcessedHandler(object sender, ProgressChangedEventArgs e);

        #endregion

        private const float MaximumJumpDistance = 14.625f;
        private static JumpRouteParameters RouteParameters;

        private static readonly Dictionary<SolarSystem, Vertex> SystemsList =
            new Dictionary<SolarSystem, Vertex>(StaticStore.Systems.Count);

        private static Dictionary<SolarSystem, List<SolarSystem>> StaticAdjacencies; //This list gets serialized 

        private readonly Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node> NodeList =
            new Dictionary<SolarSystem, FibonacciHeap<Vertex>.Node>();

        private readonly FibonacciHeap<Vertex> SystemPriorityQueue = new FibonacciHeap<Vertex>();
        private float PilotMaxRange;

        public JumpRouter()
        {
            //StaticAdjacencies is stored in the DynamicStore. If it needs to be regenerated DynamicStore will do that.
            StaticAdjacencies = DynamicStore.Instance.UniverseGraph;

            foreach (SolarSystem currentSystem in StaticStore.Systems.Values)
            {
                Vertex newVertex = new Vertex
                                       {
                                           SolarSystem = currentSystem,
                                           Cost = float.MaxValue,
                                           LYTraveled = float.MaxValue,
                                       };
                if (!SystemsList.ContainsKey(newVertex.SolarSystem))
                    //Workaround for duplicate systems in the store :lhydaarghfist:
                    SystemsList.Add(newVertex.SolarSystem, newVertex);
            }
        }

        public event SystemProcessedHandler SystemProcessed;

        public List<Vertex> GetRoute(JumpRouteParameters jumpParameters)
        {
            RouteParameters = jumpParameters;
            RouteParameters.WayPoints.Insert(0, jumpParameters.StartSystem); //Add the start to the front
            RouteParameters.WayPoints.Add(jumpParameters.DestSystem); //Add the end to the back
            PilotMaxRange = JumpDrive.GetRange(RouteParameters.CapitalShipProfile.JDC,
                                               RouteParameters.CapitalShipProfile.Ship);

            //for each waypoint of the trip, find the shortest route between the next two waypoints and it to the overall route
            var completeRoute = new List<Vertex>();
            for (int i = 0; i < RouteParameters.WayPoints.Count - 1; i++)
            {
                ResetGraph();
                completeRoute.AddRange(GetShortestPath(RouteParameters.WayPoints[i], RouteParameters.WayPoints[i + 1]));
            }

            //The way we backtrack to incorporate hitting waypoints, we need to insert the start system to be complete.
            completeRoute.Insert(0, SystemsList[RouteParameters.StartSystem]);

            return completeRoute;
        }

        /// <summary>
        ///   Finds the shortest path for the route 'Start' to 'End'
        ///   using Dijkstras weighted graph shortest path algorithm
        /// </summary>
        /// <param name="start"></param>
        /// <param name="end"></param>
        /// <returns></returns>
        private List<Vertex> GetShortestPath(SolarSystem start, SolarSystem end)
        {
            Vertex current;
            SystemsList[start].Cost = 0;
            SystemsList[start].LYTraveled = 0;

            SystemPriorityQueue.insert(SystemsList[start], SystemsList[start].Cost);

            while (!SystemPriorityQueue.isEmpty())
            {
                current = SystemPriorityQueue.removeMin().datum;

                /* If you want to do jumpbridge routing options here would be the place to do it */


                foreach (SolarSystem adjacentID in StaticAdjacencies[current.SolarSystem])
                {
                    Vertex adjacentSystem = SystemsList[adjacentID];

                    float oldcost = SystemsList[adjacentID].Cost;
                    float rangeToNextSystem = JumpDrive.GetDistance(current.SolarSystem, adjacentID);
                    if (rangeToNextSystem > PilotMaxRange)
                        continue;

                    CostWrapper nextJump = GetCost(adjacentID, rangeToNextSystem);
                    if (current.Cost + nextJump.Cost < adjacentSystem.Cost ||
                        (current.Cost + nextJump.Cost == adjacentSystem.Cost &&
                         current.LYTraveled + rangeToNextSystem < adjacentSystem.LYTraveled))
                    {
                        /* Update the algorithm variables */
                        adjacentSystem.Cost = current.Cost + nextJump.Cost;
                        adjacentSystem.LYTraveled = current.LYTraveled + rangeToNextSystem;
                        adjacentSystem.Path = current.SolarSystem;
                        adjacentSystem.FuelCost = nextJump.FuelCost;

                        adjacentSystem.JumpType = nextJump.JumpType;
                        //Current.FuelCost = NextJump.FuelCost;
                        switch (nextJump.JumpType)
                        {
                            case JumpType.Beacon:
                                adjacentSystem.Beacon = nextJump.Beacon; //Redundant but whatever.
                                break;
                            case JumpType.Bridge:
                                //AdjacentSystem.Bridge = NextJump.Bridge; //TBD.
                                break;
                            default:
                                break;
                        }


                        if (oldcost == float.MaxValue)
                            NodeList[adjacentID] = SystemPriorityQueue.insert(adjacentSystem, adjacentSystem.Cost);
                        else
                            SystemPriorityQueue.decreaseKey(NodeList[adjacentID], adjacentSystem.Cost);
                    }
                }

                //Processed a system, raise an event.
                SystemProcessed(null, null);
            }
            //Once Dijkstras finds the path, start at the end and trace backwards

            List<Vertex> systemsRoute = new List<Vertex>();
            Vertex backTrack = SystemsList[end].Copy();

            while (backTrack.SolarSystem != start)
            {
                systemsRoute.Insert(0, backTrack);
                backTrack = SystemsList[backTrack.Path].Copy();
            }

            return systemsRoute;
        }

        private void ResetGraph()
        {
            foreach (Vertex v in SystemsList.Values)
            {
                v.Cost = float.MaxValue;
                v.LYTraveled = float.MaxValue;
            }
            NodeList.Clear();
            SystemPriorityQueue.clear();
        }

        //Based on the route RouteParameters, get the cost it would take to travel to the system.
        //This assumes that the system is within jumprange as determined by GetDistance
        private static CostWrapper GetCost(SolarSystem toSystem, float jumpDistance)
        {
            /* Start with the basic normal cyno and update it if we can find an edge with lower weight */

            CostWrapper edgeCost = new CostWrapper
                                       {
                                           Cost = EdgeWeights.JumpDefault,
                                           FuelCost = JumpDrive.GetFuel(jumpDistance, RouteParameters),
                                           JumpType = JumpType.Cyno
                                       };

            if (RouteParameters.AvoidSystems.Count > 0 && RouteParameters.AvoidSystems.Contains(toSystem))
            {
                edgeCost.Cost = EdgeWeights.JumpHigh;
                edgeCost.JumpType = JumpType.Undefined;
                return edgeCost;
            }

            if (RouteParameters.UseBeacons && DynamicStore.Instance.CynoBeacons.ContainsKey(toSystem))
            {
                /* The system we have been asked to consider has a beacon */
                edgeCost.Cost = EdgeWeights.JumpBeacon;
                edgeCost.JumpType = JumpType.Beacon;
                edgeCost.Beacon = DynamicStore.Instance.CynoBeacons[toSystem];
                edgeCost.FuelCost = JumpDrive.GetFuel(jumpDistance, RouteParameters);
            }

            /* Bridges go here */

            /* Cynojammers in the future */

            if (EdgeWeights.JumpStation < edgeCost.Cost && toSystem.Stations.Count > 0)
            {
                edgeCost.Cost = EdgeWeights.JumpStation;
                edgeCost.JumpType = JumpType.Cyno;
                edgeCost.FuelCost = JumpDrive.GetFuel(jumpDistance, RouteParameters);
            }

            return edgeCost;
        }

        //Build the List of adjacencies that describes the graph/universe
        public static Dictionary<SolarSystem, List<SolarSystem>> BuildGraph()
        {
            StaticAdjacencies = new Dictionary<SolarSystem, List<SolarSystem>>();

            foreach (SolarSystem currentSystem in StaticStore.Systems.Values)
            {
                StaticAdjacencies.Add(currentSystem, new List<SolarSystem>());
                Constellation currentConstellation = Constellation.GetByID(currentSystem.ConstellationID);
                foreach (int adjConstID in currentConstellation.JumpAdjacentConstellations)
                {
                    Constellation nextConst = Constellation.GetByID(adjConstID);
                    foreach (SolarSystem possibleConnection in nextConst.Systems)
                    {
                        if (possibleConnection.Security > 0.45f)
                            continue;
                        if (JumpDrive.GetDistance(currentSystem, possibleConnection) < MaximumJumpDistance)
                            StaticAdjacencies[currentSystem].Add(possibleConnection);
                    }
                }
            }
            return StaticAdjacencies;
        }
    }
}