using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Mime;
using System.Threading;
using libEveMapNG.Data.Static;
using libEveMapNG.Types;

namespace libEveMapNG.Utility
{
    /// <summary>
    ///   A small web sever that shows gate and jump routes.  Itended to be used in the EVE IGB.
    /// 
    ///   Requires Windows XP SP2 or newer.
    /// </summary>
    public class RouteWebServer : IDisposable
    {
        /// <summary>
        ///   The address the server listens on.
        /// </summary>
        public static readonly string ServerAddress = @"http://localhost:23455/";

        private readonly HttpListener _listener;
        private readonly Thread _pumper;
        private bool _isRunning;

        /// <summary>
        ///   Creates a new instance of RouteWebServer and starts listening.  Silent does not start listening
        ///   if
        ///   <see cref="HttpListener.IsSupported" />
        ///   returns false or if the another program is using that URL.
        /// </summary>
        public RouteWebServer()
        {
            if (!HttpListener.IsSupported)
                return;

            _listener = new HttpListener();
            _listener.Prefixes.Add(ServerAddress);

            try
            {
                _listener.Start();
            }
            catch
            {
                // This probably means there is another copy of EVE Map already running.
                return;
            }

            _isRunning = true;

            _pumper = new Thread(Pump);
            _pumper.Start();
        }

        /// <summary>
        ///   The gate route to be returned by the web server.
        /// </summary>
        public IList<Vertex> GateRoute { get; set; }


        /// <summary>
        ///   The jump route to be show by the web server.
        /// </summary>
        public IList<Vertex> JumpRoute { get; set; }

        /// <summary>
        ///   Indicates whether or not the server is running.
        /// </summary>
        public bool IsRunning
        {
            get { return (_listener != null && _listener.IsListening); }
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (_listener == null)
                return;

            _isRunning = false;
            _listener.Close();
        }

        #endregion

        /// <summary>
        ///   Gets and responds to the requests.
        /// </summary>
        private void Pump()
        {
            var methods = new Dictionary<string, Action<HttpListenerRequest, HttpListenerResponse>>();
            methods.Add("/", RedirectToRoutes);
            methods.Add("/routes", UntrustedRoutes);
            methods.Add("/trustedroutes", TrustedRoutes);
            methods.Add("/requesttrust", RequestTrust);

            while (_isRunning)
            {
                HttpListenerContext con = null;
                try
                {
                    con = _listener.GetContext();
                }
                catch
                {
                    //Eat the exception
                }

                if (con != null)
                {
                    HttpListenerRequest req = con.Request;
                    HttpListenerResponse res = con.Response;
                    res.ContentType = MediaTypeNames.Text.Html;

                    if (methods.ContainsKey(req.Url.AbsolutePath))
                        methods[req.Url.AbsolutePath](req, res);
                    else
                        res.Redirect("/");


                    res.OutputStream.Close();
                    res.Close();
                }
            }
        }

        private static void RedirectToRoutes(HttpListenerRequest req, HttpListenerResponse res)
        {
            res.Redirect("/routes");
        }

        private static void RedirectToTrustedRoutes(HttpListenerRequest req, HttpListenerResponse res)
        {
            res.Redirect("/trustedroutes");
        }

        private void RequestTrust(HttpListenerRequest req, HttpListenerResponse res)
        {
            if (IsTrusted(req))
                RedirectToTrustedRoutes(req, res);
            res.AddHeader("eve.trustMe",
                          string.Format("http://{0}/::This site needs trust for advanced features.",
                                        req.Url.Host + ":" + req.Url.Port));


            StreamWriter sw = new StreamWriter(res.OutputStream);
            sw.WriteLine("<html><head><title>Eve-Map NG Routes</title></head><body>");
            sw.WriteLine("<h1>EVE-Map NG Routes</h1>");
            sw.WriteLine("<p>If you are seeing this message you failed to give trust to this website.</p>");
            sw.WriteLine("<p>Please manually grant trust through the Options -> Trusted Sites menu.</p>");
            sw.WriteLine("</body></html>");
            sw.Close();
        }

        private void TrustedRoutes(HttpListenerRequest req, HttpListenerResponse res)
        {
            if (!IsTrusted(req))
            {
                RedirectToRoutes(req, res);
                return;
            }

            string solarSystemName = req.Headers["Eve.Solarsystemname"];

            WriteRoutes(req, res, solarSystemName);
        }

        private void UntrustedRoutes(HttpListenerRequest req, HttpListenerResponse res)
        {
            if (IsTrusted(req))
            {
                RedirectToTrustedRoutes(req, res);
                return;
            }

            WriteRoutes(req, res, string.Empty);
        }

        private void WriteRoutes(HttpListenerRequest req, HttpListenerResponse res, string solarSystemName)
        {
            StreamWriter sw = new StreamWriter(res.OutputStream);

            sw.WriteLine("<html><head><title>EVE-Map NG Routes</title></head><body>");
            sw.WriteLine("<body><h1>EVE-Map NG Routes</h1>");

            //see if they are using the IGB and offer to enable special features.
            if (IsIGB(req))
            {
                if (!IsTrusted(req))
                    sw.WriteLine("<p><a href=\"/requesttrust\">Trust this website</a> to enable advanced features.</p>");
            }
            else
            {
                sw.WriteLine(
                    "<p><font color=\"red\">This site is more useful when viewed in the EVE in-game broswer.</font></p>");
            }

            if (GateRoute == null && JumpRoute == null)
            {
                sw.WriteLine(
                    "<p>There are no routes.  Plot a gate route or jump route in EVE-Map and it will show up here.</p>");
            }
            if (GateRoute != null)
                WriteGateRoute(sw, solarSystemName);
            if (JumpRoute != null)
                WriteJumpRoute(sw, solarSystemName);

            sw.WriteLine("</body></html>");
            sw.Close();
        }

        private void WriteGateRoute(TextWriter sw, string currentSystemName)
        {
            sw.WriteLine("<h2>Gate Route</h2>");
            sw.WriteLine("<table border=\"1\" cellspacing=\"0\">");
            sw.WriteLine("<tr><th>From</th> <th>P-M</th> <th>To</th> <th>Jump#</th> <th>Ozone</th></tr>");


            int jumpCount = 1;
            int ozoneUsed = 0;
            int currentVertexIndex = int.MinValue;

            for (int i = 0; i < GateRoute.Count - 1; i++)
            {
                SolarSystem current = GateRoute[i].SolarSystem;
                if (currentSystemName == current.Name)
                {
                    currentVertexIndex = i;
                    break;
                }
            }

            for (int i = 0; i < GateRoute.Count - 1; i++)
            {
                SolarSystem current = GateRoute[i].SolarSystem;
                SolarSystem next = GateRoute[i + 1].SolarSystem;
                string style = "";

                if (i < currentVertexIndex)
                    style = "background-color: green;";

                sw.WriteLine("<tr style=\"{0}\">", style);
                sw.WriteLine("<td><a href=\"showinfo:5//{1}\">{0}</a></td>", current.Name, current.ID);
                if (null != GateRoute[i + 1].Bridge.Pos)
                {
                    sw.WriteLine("<td>{0}-{1}</td>", GateRoute[i + 1].Bridge.Pos.Planet,
                                 GateRoute[i + 1].Bridge.Pos.Moon);
                    sw.WriteLine("<td><a href=\"showinfo:5//{1}\">{0}</a></td>", next.Name, next.ID);
                    sw.WriteLine("<td>{0}</td>", jumpCount);
                    sw.WriteLine("<td>{0}</td>", GateRoute[i + 1].FuelCost);

                    ozoneUsed += GateRoute[i + 1].FuelCost;
                }
                else
                {
                    sw.WriteLine("<td></td>");
                    sw.WriteLine("<td><a href=\"showinfo:5//{1}\">{0}</a></td>", next.Name, next.ID);
                    sw.WriteLine("<td>{0}</td>", jumpCount);
                    sw.WriteLine("<td></td>");
                }


                sw.WriteLine("</tr>");
                jumpCount++;
            }

            sw.WriteLine("<tr><td>Total Ozone:</td>");
            sw.WriteLine("<td></td> <td> </td> <td> </td>");
            sw.WriteLine("<td>{0}</td></tr>", ozoneUsed);

            sw.WriteLine("</table>");
        }

        private void WriteJumpRoute(TextWriter sw, string currentSystemName)
        {
            sw.WriteLine("<h2>Jump Route</h2>");
            sw.WriteLine("<table border=\"1\" cellspacing=\"0\">");
            sw.WriteLine("<tr><th>From</th> <th>To</th> <th>Distance</th> <th>Fuel</th> <th>Type</th></tr>");

            int totalFuel = 0;
            float totalLightYears = 0;
            int currentVertexIndex = int.MinValue;

            for (int i = 0; i < JumpRoute.Count - 1; i++)
            {
                SolarSystem current = (JumpRoute[i].SolarSystem);
                if (currentSystemName == current.Name)
                {
                    currentVertexIndex = i;
                    break;
                }
            }


            for (int i = 0; i < JumpRoute.Count - 1; i++)
            {
                SolarSystem currentSystem = (JumpRoute[i].SolarSystem);
                SolarSystem nextSystem = (JumpRoute[i + 1].SolarSystem);
                Vertex currentVertex = JumpRoute[i];
                Vertex nextVertex = JumpRoute[i + 1];
                string style = "";

                if (i < currentVertexIndex)
                    style = "background-color: green;";

                sw.WriteLine("<tr style=\"{0}\">", style);
                sw.WriteLine("<td><a href=\"showinfo:5//{1}\">{0}</a></td>", currentSystem.Name, currentSystem.ID);
                sw.WriteLine("<td><a href=\"showinfo:5//{1}\">{0}</a></td>", nextSystem.Name, nextSystem.ID);
                sw.WriteLine("<td>{0}</td>", JumpDrive.GetDistance(currentSystem, nextSystem));
                sw.WriteLine("<td>{0}</td>", nextVertex.FuelCost);
                sw.WriteLine("<td>{0}</td>", nextVertex.JumpType);
                sw.WriteLine("</tr>");

                totalFuel += nextVertex.FuelCost;
                totalLightYears += JumpDrive.GetDistance(currentSystem, nextSystem);
            }


            sw.WriteLine("<tr>");
            sw.WriteLine("<td>Totals:</td>");
            sw.WriteLine("<td></td>");
            sw.WriteLine("<td>{0}</td>", totalLightYears);
            sw.WriteLine("<td>{0}</td>", totalFuel);
            sw.WriteLine("<td>{0}</td>", JumpRoute.Count - 1);
            sw.WriteLine("</tr>");

            sw.WriteLine("</table>");
        }

        private bool IsIGB(HttpListenerRequest req)
        {
            if (!string.IsNullOrEmpty(req.UserAgent))
                return req.UserAgent.StartsWith("EVE-minibrowser/", StringComparison.OrdinalIgnoreCase);
            return false;
        }

        private bool IsTrusted(HttpListenerRequest req)
        {
            if (!req.Headers.AllKeys.Contains("Eve.Trusted"))
                return false;
            return req.Headers["Eve.Trusted"] == "yes";
        }
    }
}